@@ -1,10 +1,11 @@
-/* $Id: audio.h,v 3.15 1995/01/11 19:13:24 bert Exp $
+/* $Id: audio.h,v 3.21 1997/11/27 20:09:02 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -183,4 +184,8 @@ static char    *soundNames[] =
 };
 
 #endif						/* _CAUDIO_C_ */
+
+extern	void audioInit(char* display);
+extern	int	Handle_audio(int type, int volume);
+
 #endif						/* _audio_h */
