@@ -1,10 +1,11 @@
-/* $Id: defaults.h,v 3.13 1995/01/11 19:27:32 bert Exp $
+/* $Id: defaults.h,v 3.20 1997/11/27 20:09:12 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,13 +46,15 @@ enum valType {
 };
 
 typedef struct {
-    char *name;
-    char *commandLineOption;
-    char *defaultValue;
-    void *variable;
-    enum valType type;
-    void (*tuner)(void);
-    char *helpLine;
+    const char		*name;
+    const char		*commandLineOption;
+    const char		*defaultValue;
+    void		*variable;
+    enum valType	type;
+    void		(*tuner)(void);
+    const char		*helpLine;
 } optionDesc;
 
+optionDesc*	findOption(const char* name);
+
 #endif
