@@ -1,10 +1,11 @@
-/* $Id: frame.c,v 3.65 1996/05/02 16:05:46 bert Exp $
+/* $Id: frame.c,v 3.77 1998/01/08 19:28:46 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +22,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifdef	_WINDOWS
+#include "../contrib/NT/xpilots/winServer.h"
+#include <time.h>
+#include <limits.h>
+#else
 #include <unistd.h>
 #include <sys/types.h>
-#ifndef  VMS
+#if !defined(VMS)
 #include <sys/param.h>
 #endif
 #include <stdlib.h>
@@ -31,6 +37,7 @@
 #include <stdio.h>
 #include <math.h>
 #include <time.h>
+#endif
 
 #define SERVER
 #include "version.h"
@@ -47,7 +54,7 @@ char frame_version[] = VERSION;
 
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: frame.c,v 3.65 1996/05/02 16:05:46 bert Exp $";
+    "@(#)$Id: frame.c,v 3.77 1998/01/08 19:28:46 bert Exp $";
 #endif
 
 
@@ -91,19 +98,17 @@ static int		view_width,
 			debris_colors,
 			spark_rand;
 static debris_t		*debris_ptr[DEBRIS_TYPES];
-static int		debris_num[DEBRIS_TYPES],
+static unsigned int		debris_num[DEBRIS_TYPES],
 			debris_max[DEBRIS_TYPES];
 static debris_t		*fastshot_ptr[DEBRIS_TYPES * 2];
-static int		fastshot_num[DEBRIS_TYPES * 2],
+static unsigned int		fastshot_num[DEBRIS_TYPES * 2],
 			fastshot_max[DEBRIS_TYPES * 2];
 
-static int inview(float x, float y)
-{
-    return ((x > pv.world.x && x < pv.world.x + view_width)
-	    || (x > pv.realWorld.x && x < pv.realWorld.x + view_width))
-	&& ((y > pv.world.y && y < pv.world.y + view_height)
-	    || (y > pv.realWorld.y && y < pv.realWorld.y + view_height));
-}
+#define inview(x_, y_) 								\
+    (   (   ((x_) > pv.world.x && (x_) < pv.world.x + view_width)		\
+	 || ((x_) > pv.realWorld.x && (x_) < pv.realWorld.x + view_width))	\
+     && (   ((y_) > pv.world.y && (y_) < pv.world.y + view_height)		\
+	 || ((y_) > pv.realWorld.y && (y_) < pv.realWorld.y + view_height)))
 
 static int block_inview(block_visibility_t *bv, int x, int y)
 {
@@ -113,19 +118,15 @@ static int block_inview(block_visibility_t *bv, int x, int y)
 	    || (y > bv->realWorld.y && y < bv->realWorld.y + vertical_blocks));
 }
 
-#define DEBRIS_STORE(xf,yf,color,offset) \
-    int			i,						  \
-			xd,						  \
-			yd;						  \
-    if (xf < 0) {							  \
-	xf += World.width;						  \
+#define DEBRIS_STORE(xd,yd,color,offset) \
+    int			i;						  \
+    if (xd < 0) {							  \
+	xd += World.width;						  \
     }									  \
-    if (yf < 0) {							  \
-	yf += World.height;						  \
+    if (yd < 0) {							  \
+	yd += World.height;						  \
     }									  \
-    xd = (int) (xf + 0.5);						  \
-    yd = (int) (yf + 0.5);						  \
-    if ((unsigned) xd >= view_width || (unsigned) yd >= view_height) {	  \
+    if ((unsigned) xd >= (unsigned)view_width || (unsigned) yd >= (unsigned)view_height) {	  \
 	/*								  \
 	 * There's some rounding error or so somewhere.			  \
 	 * Should be possible to resolve it.				  \
@@ -137,45 +138,45 @@ static int block_inview(block_visibility_t *bv, int x, int y)
 	+ (((yd >> 8) % debris_y_areas) * debris_x_areas)		  \
 	+ ((xd >> 8) % debris_x_areas);					  \
 									  \
-    if (num >= 255) {							  \
+    if (num_ >= 255) {							  \
 	return;								  \
     }									  \
-    if (num >= max) {							  \
-	if (num == 0) {							  \
-	    ptr = (debris_t *) malloc((max = 16) * sizeof(*ptr));	  \
+    if (num_ >= max_) {							  \
+	if (num_ == 0) {							  \
+	    ptr_ = (debris_t *) malloc((max_ = 16) * sizeof(*ptr_));	  \
 	} else {							  \
-	    ptr = (debris_t *) realloc(ptr, (max += max) * sizeof(*ptr)); \
+	    ptr_ = (debris_t *) realloc(ptr_, (max_ += max_) * sizeof(*ptr_)); \
 	}								  \
-	if (ptr == 0) {							  \
+	if (ptr_ == 0) {							  \
 	    error("No memory for debris");				  \
-	    num = 0;							  \
+	    num_ = 0;							  \
 	    return;							  \
 	}								  \
     }									  \
-    ptr[num].x = (unsigned char) xd;					  \
-    ptr[num].y = (unsigned char) yd;					  \
-    num++;
+    ptr_[num_].x = (unsigned char) xd;					  \
+    ptr_[num_].y = (unsigned char) yd;					  \
+    num_++;
 
-static void fastshot_store(float xf, float yf, int color, int offset)
+static void fastshot_store(int xf, int yf, int color, int offset)
 {
-#define ptr		(fastshot_ptr[i])
-#define num		(fastshot_num[i])
-#define max		(fastshot_max[i])
+#define ptr_		(fastshot_ptr[i])
+#define num_		(fastshot_num[i])
+#define max_		(fastshot_max[i])
     DEBRIS_STORE(xf, yf, color, offset);
-#undef ptr
-#undef num
-#undef max
+#undef ptr_
+#undef num_
+#undef max_
 }
 
-static void debris_store(float xf, float yf, int color)
+static void debris_store(int xf, int yf, int color)
 {
-#define ptr		(debris_ptr[i])
-#define num		(debris_num[i])
-#define max		(debris_max[i])
+#define ptr_		(debris_ptr[i])
+#define num_		(debris_num[i])
+#define max_		(debris_max[i])
     DEBRIS_STORE(xf, yf, color, 0);
-#undef ptr
-#undef num
-#undef max
+#undef ptr_
+#undef num_
+#undef max_
 }
 
 static void fastshot_end(int conn)
@@ -268,9 +269,9 @@ static int Frame_status(int conn, int ind)
 		|| inview(Players[lock_ind]->pos.x, Players[lock_ind]->pos.y))
 	    && pl->lock.distance != 0) {
 	    SET_BIT(pl->lock.tagged, LOCK_VISIBLE);
-	    lock_dir = Wrap_findDir((int)(Players[lock_ind]->pos.x - pl->pos.x),
+	    lock_dir = (int)Wrap_findDir((int)(Players[lock_ind]->pos.x - pl->pos.x),
 				    (int)(Players[lock_ind]->pos.y - pl->pos.y));
-	    lock_dist = pl->lock.distance;
+	    lock_dist = (int)pl->lock.distance;
 	}
     }
 
@@ -357,8 +358,8 @@ static void Frame_map(int conn, int ind)
 			conn_bit = (1 << conn);
     block_visibility_t	bv;
 
-    x = pl->pos.x / BLOCK_SZ;
-    y = pl->pos.y / BLOCK_SZ;
+    x = pl->pos.bx;
+    y = pl->pos.by;
     bv.world.x = x - (horizontal_blocks >> 1);
     bv.world.y = y - (vertical_blocks >> 1);
     bv.realWorld = bv.world;
@@ -422,13 +423,13 @@ static void Frame_shots(int conn, int ind)
 
     for (i = 0; i < NumObjs; i++) {
 	shot = Obj[i];
-	if (!inview(shot->pos.x, shot->pos.y)) {
+	x = shot->pos.x;
+	y = shot->pos.y;
+	if (!inview(x, y)) {
 	    continue;
 	}
-	x = (int) (shot->pos.x + 0.5);
-	y = (int) (shot->pos.y + 0.5);
 	if ((color = shot->color) == BLACK) {
-	    printf("black %d,%d\n", shot->type, shot->id);
+	    xpprintf("black %d,%d\n", shot->type, shot->id);
 	    color = WHITE;
 	}
 	switch (shot->type) {
@@ -470,8 +471,8 @@ static void Frame_shots(int conn, int ind)
 		}
 	    }
 
-	    debris_store(shot->pos.x - pv.world.x,
-			 shot->pos.y - pv.world.y,
+	    debris_store((int)(shot->pos.x - pv.world.x),
+			 (int)(shot->pos.y - pv.world.y),
 			 color);
 	    break;
 	case OBJ_SHOT:
@@ -487,8 +488,8 @@ static void Frame_shots(int conn, int ind)
 		teamshot = 0;
 	    }
 
-	    fastshot_store(shot->pos.x - pv.world.x,
-			   shot->pos.y - pv.world.y,
+	    fastshot_store((int)(shot->pos.x - pv.world.x),
+			   (int)(shot->pos.y - pv.world.y),
 			   color, teamshot);
 	    break;
 
@@ -551,7 +552,7 @@ static void Frame_ships(int conn, int ind)
 			*pl_i;
     pulse_t		*pulse;
     int			i, j, color, dir;
-    float		x, y;
+    DFLOAT		x, y;
 
     for (i = 0; i < NumPlayers; i++) {
 	pl_i = Players[i];
@@ -613,36 +614,31 @@ static void Frame_ships(int conn, int ind)
 	    } else {
 		color = RED;
 	    }
-	    Send_laser(conn,
-		color,
-		(int) (x + 0.5f),
-		(int) (y + 0.5f),
-		pulse->len,
-		dir);
+	    Send_laser(conn, color, (int)x, (int)y, pulse->len, dir);
 	}
 	if (!inview(pl_i->pos.x, pl_i->pos.y)) {
 	    continue;
 	}
 	if (BIT(pl_i->status, PAUSE)) {
 	    Send_paused(conn,
-		(int) (pl_i->pos.x + 0.5),
-		(int) (pl_i->pos.y + 0.5),
-		pl_i->count);
+			pl_i->pos.x,
+			pl_i->pos.y,
+			pl_i->count);
 	    continue;
 	}
 	for (j = 0; j < pl_i->ecmInfo.count; j++) {
 	    Send_ecm(conn,
-		     (int) (pl_i->ecmInfo.pos[j].x + 0.5f),
-		     (int) (pl_i->ecmInfo.pos[j].y + 0.5f),
+		     (int)pl_i->ecmInfo.pos[j].x,
+		     (int)pl_i->ecmInfo.pos[j].y,
 		     pl_i->ecmInfo.size[j]);
 	}
 	if (pl_i->transInfo.count) {
 	    player *pl = Players[GetInd[pl_i->transInfo.pl_id]];
 	    Send_trans(conn,
-		       (int) (pl->pos.x + 0.5),
-		       (int) (pl->pos.y + 0.5),
-		       (int) (pl_i->pos.x + 0.5),
-		       (int) (pl_i->pos.y + 0.5));
+		       pl->pos.x,
+		       pl->pos.y,
+		       pl_i->pos.x,
+		       pl_i->pos.y);
 	}
 
 	/* Don't transmit information is fighter is invisible */
@@ -653,35 +649,31 @@ static void Frame_ships(int conn, int ind)
 	     * Transmit ship information
 	     */
 	    Send_ship(conn,
-		(int) (pl_i->pos.x + 0.5),
-		(int) (pl_i->pos.y + 0.5),
-		pl_i->id,
-		pl_i->dir,
-		BIT(pl_i->used, OBJ_SHIELD) != 0,
-		BIT(pl_i->used, OBJ_CLOAKING_DEVICE) != 0,
-		BIT(pl_i->used, OBJ_EMERGENCY_SHIELD) != 0
+		      pl_i->pos.x,
+		      pl_i->pos.y,
+		      pl_i->id,
+		      pl_i->dir,
+		      BIT(pl_i->used, OBJ_SHIELD) != 0,
+		      BIT(pl_i->used, OBJ_CLOAKING_DEVICE) != 0,
+		      BIT(pl_i->used, OBJ_EMERGENCY_SHIELD) != 0
 	    );
 	}
 	if (BIT(pl_i->used, OBJ_REFUEL)) {
 	    if (inview(World.fuel[pl_i->fs].pix_pos.x,
 		       World.fuel[pl_i->fs].pix_pos.y)) {
 		Send_refuel(conn,
-		    (int) (World.fuel[pl_i->fs].pix_pos.x),
-		    (int) (World.fuel[pl_i->fs].pix_pos.y),
-		    (int) (pl_i->pos.x + 0.5),
-		    (int) (pl_i->pos.y + 0.5));
+			    (int)World.fuel[pl_i->fs].pix_pos.x,
+			    (int)World.fuel[pl_i->fs].pix_pos.y,
+			    pl_i->pos.x,
+			    pl_i->pos.y);
 	    }
 	}
 	if (BIT(pl_i->used, OBJ_REPAIR)) {
-	    float x = World.targets[pl_i->repair_target].pos.x
-		* BLOCK_SZ+BLOCK_SZ/2;
-	    float y = World.targets[pl_i->repair_target].pos.y
-		* BLOCK_SZ+BLOCK_SZ/2;
+	    DFLOAT x = (DFLOAT)(World.targets[pl_i->repair_target].pos.x + 0.5) * BLOCK_SZ;
+	    DFLOAT y = (DFLOAT)(World.targets[pl_i->repair_target].pos.y + 0.5) * BLOCK_SZ;
 	    if (inview(x, y)) {
 		/* same packet as refuel */
-		Send_refuel(conn,
-		    (int) (pl_i->pos.x + 0.5), (int) (pl_i->pos.y + 0.5),
-		    (int) (x + 0.5), (int) (y + 0.5));
+		Send_refuel(conn, pl_i->pos.x, pl_i->pos.y, (int) x, (int) y);
 	    }
 	}
 	if (pl_i->tractor != NULL
@@ -692,30 +684,30 @@ static void Frame_ships(int conn, int ind)
 
 	    for (j = 0; j < 3; j++) {
 		Send_connector(conn,
-		       (int) (t->pos.x + t->ship->pts[j][t->dir].x + 0.5),
-		       (int) (t->pos.y + t->ship->pts[j][t->dir].y + 0.5),
-		       (int) (pl_i->pos.x + 0.5),
-		       (int) (pl_i->pos.y + 0.5), 1);
+			       (int)(t->pos.x + t->ship->pts[j][t->dir].x),
+			       (int)(t->pos.y + t->ship->pts[j][t->dir].y),
+			       pl_i->pos.x,
+			       pl_i->pos.y, 1);
 	    }
 	}
 
 	if (pl_i->ball != NULL
 	    && inview(pl_i->ball->pos.x, pl_i->ball->pos.y)) {
 	    Send_connector(conn,
-		(int) (pl_i->ball->pos.x + 0.5),
-		(int) (pl_i->ball->pos.y + 0.5),
-		(int) (pl_i->pos.x + 0.5),
-		(int) (pl_i->pos.y + 0.5), 0);
+			   pl_i->ball->pos.x,
+			   pl_i->ball->pos.y,
+			   pl_i->pos.x,
+			   pl_i->pos.y, 0);
 	}
     }
 }
 
 static void Frame_radar(int conn, int ind)
 {
-    register int	i, mask, shownuke, s;
+    int			i, mask, shownuke, s;
     player		*pl = Players[ind];
     object		*shot;
-    float		x, y;
+    DFLOAT		x, y;
 
 #ifndef NO_SMART_MIS_RADAR
     if (nukesOnRadar) {
@@ -759,7 +751,7 @@ static void Frame_radar(int conn, int ind)
 	    y = shot->pos.y;
 	    if (Wrap_length(pl->pos.x - x,
 			    pl->pos.y - y) <= pl->sensor_range) {
-		Send_radar(conn, (int) (x + 0.5), (int) (y + 0.5), s);
+		Send_radar(conn, (int)x, (int)y, s);
 	    }
 	}
     }
@@ -792,7 +784,7 @@ static void Frame_radar(int conn, int ind)
 		&& frame_loops % 5 >= 3) {
 		continue;
 	    }
-	    Send_radar(conn, (int) (x + 0.5), (int) (y + 0.5), 3);
+	    Send_radar(conn, (int)x, (int)y, 3);
 	}
     }
 }
@@ -809,8 +801,8 @@ static void Frame_parameters(int conn, int ind)
     horizontal_blocks = (view_width + (BLOCK_SZ - 1)) / BLOCK_SZ;
     vertical_blocks = (view_height + (BLOCK_SZ - 1)) / BLOCK_SZ;
 
-    pv.world.x = (int)(pl->pos.x + 0.5) - view_width / 2;	/* Scroll */
-    pv.world.y = (int)(pl->pos.y + 0.5) - view_height / 2;
+    pv.world.x = pl->pos.x - view_width / 2;	/* Scroll */
+    pv.world.y = pl->pos.y - view_height / 2;
     pv.realWorld = pv.world;
     if (BIT (World.rules->mode, WRAP_PLAY)) {
 	if (pv.world.x < 0 && pv.world.x + view_width < World.width) {
@@ -865,7 +857,7 @@ void Frame_update(void)
 	     * Lower the frame rate for non-playing players
 	     * to reduce network load.
 	     */
-	    if (BIT(pl->status, PAUSE)
+	    if ((BIT(pl->status, PAUSE))
 		? ((frame_loops & 0x03) != 0)
 		: ((frame_loops & 0x01) != 0)) {
 		continue;
@@ -877,7 +869,7 @@ void Frame_update(void)
 	*/
 	if (pl->player_count > 0) {
 	    pl->player_round++;
-	    if (pl->player_round > pl->player_count) {
+	    if (pl->player_round >= pl->player_count) {
 		pl->player_round = 0;
 		continue;
 	    }
@@ -901,10 +893,11 @@ void Frame_update(void)
 	 */
 	if (BIT(pl->lock.tagged, LOCK_PLAYER)) {
 	    if ((BIT(pl->status, (GAME_OVER|PLAYING)) == (GAME_OVER|PLAYING))
-		|| (BIT(pl->status, PAUSE)
-		    && BIT(World.rules->mode, TEAM_PLAY)
-		    && pl->team != TEAM_NOT_SET
-		    && pl->team == Players[GetInd[pl->lock.pl_id]]->team)) {
+		|| (BIT(pl->status, PAUSE) &&
+		    ((BIT(World.rules->mode, TEAM_PLAY)
+		     && pl->team != TEAM_NOT_SET
+		     && pl->team == Players[GetInd[pl->lock.pl_id]]->team)
+		    ))) {
 		ind = GetInd[pl->lock.pl_id];
 	    } else {
 		ind = i;
@@ -914,7 +907,6 @@ void Frame_update(void)
 	}
 	if (Players[ind]->damaged > 0) {
 	    Send_damaged(conn, Players[ind]->damaged);
-	    Players[ind]->damaged--;
 	} else {
 	    Frame_parameters(conn, ind);
 	    if (Frame_status(conn, ind) <= 0) {
@@ -940,12 +932,12 @@ void Frame_update(void)
     oldTimeLeft = newTimeLeft;
 }
 
-void Set_message(char *message)
+void Set_message(const char *message)
 {
     player		*pl;
     int			i;
-    char		*msg,
-			tmp[MSG_LEN];
+    const char		*msg;
+    char		tmp[MSG_LEN];
 
     if ((i = strlen(message)) >= MSG_LEN) {
 #ifndef SILENT
