@@ -1,10 +1,11 @@
-/* $Id: netserver.c,v 3.124 1996/05/13 12:16:50 bert Exp $
+/* $Id: netserver.c,v 3.143 1998/01/08 19:28:48 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -88,6 +89,14 @@
  * if the acknowledgement timer expires.
  */
 
+#ifdef	_WINDOWS
+#include "../contrib/NT/xpilots/winServer.h"
+#include <io.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <time.h>
+#else
+
 #include "types.h"
 #include <unistd.h>
 #include <stdlib.h>
@@ -96,7 +105,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <sys/stat.h>
-#ifndef  VMS
+#if !defined(VMS)
 # include <sys/param.h>
 #endif
 #if defined(__hpux)
@@ -111,11 +120,22 @@
 # ifdef sco
 #  include <time.h>
 # endif
+#ifndef	_WINDOWS
 # include <sys/time.h>
 #endif
+#endif
+#ifdef	_WINDOWS
+#include "winNet.h"
+#include <io.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#else
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#endif
+#endif
+
 #include <ctype.h>
 
 #define SERVER
@@ -236,6 +256,29 @@ static int Init_setup(void)
 	for (y = 0; y < World.y; y++, mapptr++) {
 	    type = World.block[x][y];
 	    switch (type) {
+	    case ACWISE_GRAV:
+	    case CWISE_GRAV:
+	    case POS_GRAV:
+	    case NEG_GRAV:
+	    case UP_GRAV:
+	    case DOWN_GRAV:
+	    case RIGHT_GRAV:
+	    case LEFT_GRAV:
+		if (!gravityVisible)
+		    type = SETUP_SPACE;
+		break;
+	    case WORMHOLE:
+		if (!wormholeVisible)
+		    type = SETUP_SPACE;
+		break;
+	    case ITEM_CONCENTRATOR:
+		if (!itemConcentratorVisible)
+		    type = SETUP_SPACE;
+		break;
+	    default:
+		break;
+	    }
+	    switch (type) {
 	    case SPACE:		*mapptr = SETUP_SPACE; break;
 	    case FILLED:	*mapptr = SETUP_FILLED; break;
 	    case REC_RU:	*mapptr = SETUP_REC_RU; break;
@@ -247,6 +290,10 @@ static int Init_setup(void)
 	    case CWISE_GRAV:	*mapptr = SETUP_CWISE_GRAV; break;
 	    case POS_GRAV:	*mapptr = SETUP_POS_GRAV; break;
 	    case NEG_GRAV:	*mapptr = SETUP_NEG_GRAV; break;
+	    case UP_GRAV:	*mapptr = SETUP_UP_GRAV; break;
+	    case DOWN_GRAV:	*mapptr = SETUP_DOWN_GRAV; break;
+	    case RIGHT_GRAV:	*mapptr = SETUP_RIGHT_GRAV; break;
+	    case LEFT_GRAV:	*mapptr = SETUP_LEFT_GRAV; break;
 	    case ITEM_CONCENTRATOR:	*mapptr = SETUP_ITEM_CONCENTRATOR; break;
 	    case DECOR_FILLED:	*mapptr = SETUP_DECOR_FILLED; break;
 	    case DECOR_RU:	*mapptr = SETUP_DECOR_RU; break;
@@ -341,7 +388,7 @@ static int Init_setup(void)
 
 #ifndef SILENT
     if (type != SETUP_MAP_UNCOMPRESSED) {
-	printf("Map compression ratio is %-4.2f%%\n",
+	xpprintf("%s Map compression ratio is %-4.2f%%\n", showtime(),
 	    100.0 * size / (World.x * World.y));
     }
 #endif
@@ -507,7 +554,7 @@ static void Conn_set_state(connection_t *connp, int state, int drain_state)
  * Since 3.0.6 the client receives a short message
  * explaining why the connection was terminated.
  */
-void Destroy_connection(int ind, char *reason)
+void Destroy_connection(int ind, const char *reason)
 {
     connection_t	*connp = &Conn[ind];
     int			id,
@@ -533,7 +580,7 @@ void Destroy_connection(int ind, char *reason)
 	DgramWrite(sock, pkt, len);
     }
 #ifndef SILENT
-    printf("%s: Goodbye %s=%s@%s|%s (\"%s\")\n",
+    xpprintf("%s Goodbye %s=%s@%s|%s (\"%s\")\n",
 	   showtime(),
 	   connp->nick ? connp->nick : "",
 	   connp->real ? connp->real : "",
@@ -645,9 +692,9 @@ int Setup_connection(char *real, char *nick, char *dpy, int team,
 
     if (free_conn_index >= max_connections) {
 #ifndef SILENT
-	printf("Full house for %s(%s)@%s(%s)\n", real, nick, host, dpy);
+		xpprintf("%s Full house for %s(%s)@%s(%s)\n", showtime(), real, nick, host, dpy);
 #endif
-	return -1;
+		return -1;
     }
     connp = &Conn[free_conn_index];
 
@@ -655,6 +702,8 @@ int Setup_connection(char *real, char *nick, char *dpy, int team,
 	error("Cannot create datagram socket (%d)", sl_errno);
 	return -1;
     }
+#if 0
+	/* this shouldn't be an issue with the newfangled use of select */
     if (sock >= MAX_SELECT_FD) {
 	/* Not handled with our current oldfashioned use of select(2). */
 	errno = 0;
@@ -662,6 +711,7 @@ int Setup_connection(char *real, char *nick, char *dpy, int team,
 	DgramClose(sock);
 	return -1;
     }
+#endif
     if ((my_port = GetPortNum(sock)) == 0) {
 	error("Cannot get port from socket");
 	DgramClose(sock);
@@ -771,17 +821,28 @@ static int Handle_listening(int ind)
     connp->r.len = n;
     connp->his_port = DgramLastport();
     if (DgramConnect(connp->w.sock, connp->addr, connp->his_port) == -1) {
-	error("Cannot connect datagram socket (%s,%d)",
-	      connp->host, connp->his_port);
-	Destroy_connection(ind, "connect error");
-	return -1;
+	error("Cannot connect datagram socket (%s,%d,%d)",
+	      connp->addr, connp->his_port, sl_errno);
+	if (GetSocketError(connp->w.sock)) {
+	    error("GetSocketError fails too, giving up");
+	    Destroy_connection(ind, "connect error");
+	    return -1;
+	}
+	errno = 0;
+	if (DgramConnect(connp->w.sock, connp->addr, connp->his_port) == -1) {
+	    error("Still cannot connect datagram socket (%s,%d,%d)",
+		  connp->addr, connp->his_port, sl_errno);
+	    Destroy_connection(ind, "connect error");
+	    return -1;
+	}
     }
 #ifndef SILENT
-    printf("%s: Welcome %s=%s@%s|%s (%s/%d)", showtime(), connp->nick,
+    xpprintf("%s Welcome %s=%s@%s|%s (%s/%d)", showtime(), connp->nick,
 	   connp->real, connp->host, connp->dpy, connp->addr, connp->his_port);
     if (connp->version != MY_VERSION)
-	printf(" (version %04x)", connp->version);
-    printf("\n");
+		xpprintf(" (version %04x)\n", connp->version);
+	else
+		xpprintf("\n");
 #endif
     if (connp->r.ptr[0] != PKT_VERIFY) {
 	Send_reply(ind, PKT_VERIFY, PKT_FAILURE);
@@ -798,8 +859,8 @@ static int Handle_listening(int ind)
     }
     if (strcmp(real, connp->real)) {
 #ifndef SILENT
-	printf("Client verified incorrectly (%s,%s)(%s,%s)",
-	       real, nick, connp->real, connp->nick);
+		xpprintf("%s Client verified incorrectly (%s,%s)(%s,%s)\n",
+	       showtime(), real, nick, connp->real, connp->nick);
 #endif
 	Send_reply(ind, PKT_VERIFY, PKT_FAILURE);
 	Send_reliable(ind);
@@ -903,11 +964,6 @@ static int Handle_login(int ind)
 	error("Not enough bases for players");
 	return -1;
     }
-    if (Id >= MAX_ID) {
-	errno = 0;
-	error("Id too big (%d)", Id);
-	return -1;
-    }
     if (BIT(World.rules->mode, TEAM_PLAY)) {
 	if (connp->team <= 0 || connp->team >= MAX_TEAMS) {
 	    connp->team = TEAM_NOT_SET;
@@ -940,7 +996,9 @@ static int Handle_login(int ind)
 	    return -1;
 	}
     }
-    Init_player(NumPlayers, connp->ship);
+    if (!Init_player(NumPlayers, connp->ship)) {
+	return -1;
+    }
     pl = Players[NumPlayers];
     strcpy(pl->name, connp->nick);
     strcpy(pl->realname, connp->real);
@@ -956,7 +1014,8 @@ static int Handle_login(int ind)
 	World.teams[pl->team].NumMembers++;
     }
     NumPlayers++;
-    connp->id = Id++;
+    request_ID();
+    connp->id = pl->id;
     pl->conn = ind;
     memset(pl->last_keyv, 0, sizeof(pl->last_keyv));
     memset(pl->prev_keyv, 0, sizeof(pl->prev_keyv));
@@ -969,7 +1028,7 @@ static int Handle_login(int ind)
     }
 
 #ifndef	SILENT
-    printf("%s (%d) starts at startpos %d.\n",
+    xpprintf("%s %s (%d) starts at startpos %d.\n", showtime(),
 	   pl->name, NumPlayers, pl->home_base);
 #endif
 
@@ -1187,7 +1246,7 @@ int Input(void)
 
     if (num_logins | num_logouts) {
 	/* Tell the meta server */
-	Send_meta_server(1);
+	Meta_update(1);
 	num_logins = 0;
 	num_logouts = 0;
     }
@@ -1228,7 +1287,7 @@ int Send_self(int ind,
 {
     connection_t	*connp = &Conn[ind];
     int			n;
-    u_byte		stat = status;
+    u_byte		stat = (u_byte)status;
     int			sbuf_len = connp->w.len;
 
     n = Packet_printf(&connp->w,
@@ -1448,7 +1507,7 @@ int Send_fuel(int ind, int num, int fuel)
 			 num, fuel >> FUEL_SCALE_BITS);
 }
 
-int Send_score_object(int ind, int score, int x, int y, char *string)
+int Send_score_object(int ind, int score, int x, int y, const char *string)
 {
     connection_t	*connp = &Conn[ind];
 
@@ -1668,7 +1727,7 @@ int Send_eyes(int ind, int id)
     return Packet_printf(&connp->w, "%c%hd", PKT_EYES, id);
 }
 
-int Send_message(int ind, char *msg)
+int Send_message(int ind, const char *msg)
 {
     connection_t	*connp = &Conn[ind];
 
@@ -1854,7 +1913,7 @@ static int Receive_power(int ind)
     unsigned char	ch;
     short		tmp;
     int			n;
-    float		power;
+    DFLOAT		power;
 
     if ((n = Packet_scanf(&connp->r, "%c%hd", &ch, &tmp)) <= 0) {
 	if (n == -1) {
@@ -1862,7 +1921,7 @@ static int Receive_power(int ind)
 	}
 	return n;
     }
-    power = (float) tmp / 256.0F;
+    power = (DFLOAT) tmp / 256.0F;
     pl = Players[GetInd[connp->id]];
     switch (ch) {
     case PKT_POWER:
@@ -2256,7 +2315,8 @@ static void Handle_talk(int ind, char *str)
 {
     connection_t	*connp = &Conn[ind];
     player		*pl = Players[GetInd[connp->id]];
-    int			i, sent, team, len;
+    int			i, sent, team;
+	unsigned int	len;
     char		*cp,
 			msg[MSG_LEN * 2];
 
@@ -2294,7 +2354,7 @@ static void Handle_talk(int ind, char *str)
 	FILE *fp = fopen(LOGFILE, "a");
 	if (fp) {
 	    fprintf(fp,
-		    "%s:[%s]{%s@%s(%s)|%s}:\n"
+		    "%s[%s]{%s@%s(%s)|%s}:\n"
 		    "\t%s\n",
 		    showtime(),
 		    pl->name,
@@ -2306,10 +2366,21 @@ static void Handle_talk(int ind, char *str)
 	}
     }
     else {						/* Player message */
-	for (sent = -1, i = 0; i < NumPlayers; i++) {
-	    if (strncasecmp(Players[i]->name, str, len) == 0
-		|| strncasecmp(Players[i]->realname, str, len) == 0)
-		sent = (sent == -1) ? i : -2;
+	sent = -1;
+	/* first look for an exact match on player nickname. */
+	for (i = 0; i < NumPlayers; i++) {
+	    if (strcasecmp(Players[i]->name, str) == 0) {
+		sent = i;
+		break;
+	    }
+	}
+	if (sent == -1) {
+	    /* now look for a partial match on both nick and realname. */
+	    for (sent = -1, i = 0; i < NumPlayers; i++) {
+		if (strncasecmp(Players[i]->name, str, len) == 0
+		    || strncasecmp(Players[i]->realname, str, len) == 0)
+		    sent = (sent == -1) ? i : -2;
+	    }
 	}
 	switch (sent) {
 	case -2:
@@ -2551,6 +2622,9 @@ static int Receive_motd(int ind)
  * If this MOTD buffer hasn't been accessed for a while
  * then on the next access the MOTD file is checked for changes.
  */
+#ifdef	_WINDOWS
+#define	close(__a)	_close(__a)
+#endif
 int Get_motd(char *buf, int offset, int maxlen, int *size_ptr)
 {
     static int		motd_size;
@@ -2578,7 +2652,7 @@ int Get_motd(char *buf, int offset, int maxlen, int *size_ptr)
 	    motd_size = 0;
 	    return -1;
 	}
-	if (fstat(fd, &st) == -1) {
+	if (fstat(fd, &st) == -1 || st.st_size == 0) {
 	    motd_size = 0;
 	    close(fd);
 	    return -1;
@@ -2677,7 +2751,7 @@ static int Receive_pointer_move(int ind)
     unsigned char	ch;
     short		movement;
     int			n;
-    float		turnspeed, turndir;
+    DFLOAT		turnspeed, turndir;
 
     if ((n = Packet_scanf(&connp->r, "%c%hd", &ch, &movement)) <= 0) {
 	if (n == -1) {
@@ -2690,11 +2764,11 @@ static int Receive_pointer_move(int ind)
 	Autopilot(ind, 0);
     turnspeed = movement * pl->turnspeed / MAX_PLAYER_TURNSPEED;
     if (turnspeed < 0) {
-	turndir = -1;
+	turndir = -1.0;
 	turnspeed = -turnspeed;
     }
     else {
-	turndir = 1;
+	turndir = 1.0;
     }
     LIMIT(turnspeed, MIN_PLAYER_TURNSPEED, MAX_PLAYER_TURNSPEED);
     pl->turnvel -= turndir * turnspeed;
@@ -2722,7 +2796,7 @@ static int Receive_fps_request(int ind)
 	if (fps > FPS) pl->player_fps = FPS;
 	if (fps < (FPS / 2)) pl->player_fps = FPS / 2;
 	if (fps == 0) pl->player_fps = FPS;
-	n = FPS - fps;
+	n = FPS - pl->player_fps;
 	if (n <= 0) {
 	    pl->player_count = 0;
 	} else {
