@@ -1,10 +1,11 @@
-/* $Id: robot.c,v 3.60 1996/04/07 22:44:40 bert Exp $
+/* $Id: robot.c,v 3.71 1998/01/08 19:28:51 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +23,16 @@
  */
 /* Robot code originally submitted by Maurice Abraham. */
 
+#ifdef	_WINDOWS
+#include "../contrib/NT/xpilots/winServer.h"
+#include <limits.h>
+#else
 #include <unistd.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
 #include <stdio.h>
+#endif
 
 #define SERVER
 #include "version.h"
@@ -48,7 +54,8 @@
 char robot_version[] = VERSION;
 
 #ifndef	lint
-static char sourceid[] = "@(#)robot.c,v 1.3 1992/06/26 15:25:46 bjoerns Exp";
+static char sourceid[] =
+    "@(#)robot.c,v 1.3 1992/06/26 15:25:46 bjoerns Exp";
 #endif
 
 /*
@@ -581,13 +588,13 @@ void Robot_init(void)
 
 
 static void Robot_talks(enum robot_talk_t says_what,
-			char *robot_name, char *other_name)
+			char *robot_name, const char *other_name)
 {
     /*
      * Insert your own witty messages here and remove the silly ones.
      */
 
-    static char *enter_msgs[] = {
+    static const char *enter_msgs[] = {
 	"%s just can't stand you anymore.",
 	"%s has come to give you a hard time.",
 	"%s is looking for trouble.",
@@ -602,7 +609,7 @@ static void Robot_talks(enum robot_talk_t says_what,
 	"%s has no sense of humour.",
 	"%s is back from the Sirius wars, and he's in a violent mood.",
     };
-    static char *leave_msgs[] = {
+    static const char *leave_msgs[] = {
 	"That's it, I've had enough. :(   I'm outta here. [%s]",
 	"Later people.  It's been fun. [%s]",
 	"Gotta go, ... er ... this ... er ... lab is closing. [%s]",
@@ -617,7 +624,7 @@ static void Robot_talks(enum robot_talk_t says_what,
 	"Wow, this game is just killing me. :( [%s]",
 	"I'll be back when you stop cheating! :-( [%s]",
     };
-    static char *kill_msgs[] = {
+    static const char *kill_msgs[] = {
 	"Have some %s.  Have some! [%s]",
 	"You want some more %s? [%s]",
 	"%s lost his stuff again.  That's just tooooooo bad. :) [%s]",
@@ -627,20 +634,21 @@ static void Robot_talks(enum robot_talk_t says_what,
 	"Hey %s, go play chess instead. [%s]",
 	"I think Darwin would've said you're too unfit to survive %s :) [%s]",
 	"Oh my, what colourful explosions you make %s. :) [%s]",
+	"Hey %s, maybe its time you upgraded that old 386. [%s]",
     };
-    static char *war_msgs[] = {
+    static const char *war_msgs[] = {
 	"UNBELIEVABLE, me shot down by %s?!?!  This means war [%s]",
 	"People like %s just piss me off. [%s]",
 	"Nice %s.  But now its my turn. [%s]",
 	"Red alert... target: %s. [%s]",
-	"$%#^@#$^#$%  That's the last time you did that %s! [%s]",
+	"$%#^@#$^#$%  That's the last time you do that %s! [%s]",
 	"Enough's enough!  It's only room enough for one of us %s here. [%s]",
 	"I'm sorry %s, but you must... DIE!!!!! [%s]",
 	"Jihad!  Die %s!  Die! [%s]",
     };
 
     static int		next_msg = -1;
-    char		**msgsp;
+    const char		**msgsp;
     int			two, i, n;
     char		msg[MSG_LEN];
 
@@ -700,6 +708,10 @@ static void Robot_create(void)
     robot_data_t	*data, *new_data;
     robot_type_t	*rob_type;
 
+    if (peek_ID() == 0) {
+	return;
+    }
+
     if ((new_data = (robot_data_t *)malloc(sizeof(robot_data_t))) == NULL) {
 	perror("malloc robot_data");
 	return;
@@ -752,7 +764,7 @@ static void Robot_create(void)
 
     strcpy(robot->name, rob->name);
     strcpy(robot->realname, "robot");
-    strcpy(robot->hostname, "robots.org");
+    strcpy(robot->hostname, "xpilot.org");
 
     robot->color = WHITE;
     robot->turnspeed = MAX_PLAYER_TURNSPEED;
@@ -761,6 +773,7 @@ static void Robot_create(void)
     robot->turnresistance_s = 0.12;
     robot->power = MAX_PLAYER_POWER;
     robot->power_s = MAX_PLAYER_POWER;
+    robot->check = 0;
     if (BIT(World.rules->mode, TEAM_PLAY))
 	robot->team = 0;		/* Robots are on their own team */
     if (robot->mychar != 'W')
@@ -781,9 +794,9 @@ static void Robot_create(void)
      * robot->shot_mass	= ShotsMass + (rob->defense - rob->attack) / 10.0;
      * robot->max_speed	= SPEED_LIMIT - robot->shot_speed;
      */
+    request_ID();
     NumPlayers++;
     World.teams[0].NumMembers++;
-    Id++;
     NumRobots++;
 
     for (i = 0; i < NumPlayers - 1; i++) {
@@ -796,8 +809,8 @@ static void Robot_create(void)
     Robot_talks(ROBOT_TALK_ENTER, robot->name, "");
 
 #ifndef	SILENT
-    printf("%s (%d, %s) starts at startpos %d.\n",
-	   robot->name, NumPlayers, robot->realname, robot->home_base);
+    xpprintf("%s %s (%d, %s) starts at startpos %d.\n",
+	   showtime(), robot->name, NumPlayers, robot->realname, robot->home_base);
 #endif
 
     updateScores = true;
@@ -851,12 +864,6 @@ void Robot_delete(int ind, int kicked)
 }
 
 
-void Robot_go_gome(int ind)
-{
-    (*robot_types[Players[ind]->robot_data_ptr->robot_types_ind].go_home)(ind);
-}
-
-
 /*
  * Turn on a war lock.
  */
@@ -920,17 +927,27 @@ void Robot_war(int ind, int killer)
 
     if (IS_ROBOT_PTR(kp)) {
 	Robot_talks(ROBOT_TALK_KILL, kp->name, pl->name);
+
+	if (Robot_war_on_player(killer) == pl->id)
+	    for (i = 0; i < NumPlayers; i++) {
+		if (Players[i]->conn != NOT_CONNECTED) {
+		    Send_war(Players[i]->conn, kp->id, -1);
+		}
+	    }
+	Robot_set_war(killer, -1);
     }
 
     if (IS_ROBOT_PTR(pl)
-	&& rand()%100 < kp->score - pl->score) {
+	&& rand()%100 < kp->score - pl->score
+	&& !(BIT(World.rules->mode, TEAM_PLAY) && pl->team == kp->team)) {
 
 	Robot_talks(ROBOT_TALK_WAR, pl->name, kp->name);
 
 	/*
 	 * Give fuel for offensive.
+	 * BD: unfair advantage.
 	 */
-	pl->fuel.sum = MAX_PLAYER_FUEL;
+	/* pl->fuel.sum = MAX_PLAYER_FUEL; */
 
 	if (Robot_war_on_player(ind) != kp->id) {
 	    for (i = 0; i < NumPlayers; i++) {
@@ -1026,6 +1043,13 @@ void Robot_update(void)
 	    continue;
 	}
 
+	/* Bucko sez: I had a server crash here as i was exiting play.  Seems the type_ext for me
+	   was 0 (?) so it defaulted to robot handling, which is a bad thing (robot_data_ptr == NULL)
+	   so i added this check for real robots.  I'm not sure if the Player[ind] will clean up
+	   or just leak.
+	*/
+	if (!IS_ROBOT_PTR(pl))
+		continue;
 	/*
 	 * So it is a genuine robot...
 	 * Check if it is still considered good enough to continue playing...
