@@ -1,10 +1,11 @@
-/* $Id: saudio.c,v 3.18 1995/01/11 19:56:30 bert Exp $
+/* $Id: saudio.c,v 3.25 1997/11/27 20:09:34 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,10 +23,14 @@
  */
 /* This piece of code was provided by Greg Renda (greg@ncd.com). */
 
-#define SERVER
+#ifdef	_WINDOWS
+#include <windows.h>
+#else
 #include <stdlib.h>
 #include <stdio.h>
+#endif
 
+#define SERVER
 #include "version.h"
 #include "config.h"
 #include "const.h"
@@ -138,11 +143,11 @@ void sound_play_all(int index)
  * is what the player can see on the screen. A volume is assigned to the
  * sound depending on the location within the sound range.
  */
-void sound_play_sensors(float x, float y, int index)
+void sound_play_sensors(DFLOAT x, DFLOAT y, int index)
 {
     int             i,
 		    volume;
-    float           dx,
+    DFLOAT           dx,
 		    dy,
 		    range,
 		    factor;
