@@ -1,10 +1,11 @@
-/* $Id: saudio.h,v 3.10 1995/01/11 19:56:48 bert Exp $
+/* $Id: saudio.h,v 3.16 1997/11/27 20:09:34 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,7 +52,7 @@ int		sound_player_init(player *);
 void		sound_player_onoff(player *pl, int onoff);
 void		sound_play_player(player *, int);
 void		sound_play_all(int);
-void		sound_play_sensors(float, float, int);
+void		sound_play_sensors(DFLOAT, DFLOAT, int);
 void		sound_play_queued(player * pl);
 void		sound_close(player * pl);
 
