@@ -1,10 +1,11 @@
-/* $Id: setup.h,v 3.15 1995/01/11 19:58:38 bert Exp $
+/* $Id: setup.h,v 3.23 1997/11/27 20:09:36 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -74,7 +75,21 @@
 #define SETUP_BASE_HIGHEST	69	/* highest base number */
 #define SETUP_TARGET		70	/* + team number (10) */
 #define SETUP_CHECK		80	/* + check point number (26) */
-#define SETUP_ITEM_CONCENTRATOR		110
+#define SETUP_ITEM_CONCENTRATOR	110
+#define SETUP_DECOR_FILLED	111
+#define SETUP_DECOR_RU		112
+#define SETUP_DECOR_RD		113
+#define SETUP_DECOR_LU		114
+#define SETUP_DECOR_LD		115
+#define SETUP_DECOR_DOT_FILLED	116
+#define SETUP_DECOR_DOT_RU	117
+#define SETUP_DECOR_DOT_RD	118
+#define SETUP_DECOR_DOT_LU	119
+#define SETUP_DECOR_DOT_LD	120
+#define SETUP_UP_GRAV		121
+#define SETUP_DOWN_GRAV		122
+#define SETUP_RIGHT_GRAV	123
+#define SETUP_LEFT_GRAV		124
 
 #define BLUE_UP			0x01
 #define BLUE_RIGHT		0x02
@@ -86,6 +101,14 @@
 #define BLUE_BELOW		0x40	/* when triangle is below diagonal */
 #define BLUE_BIT		0x80	/* set when drawn with blue lines */
 
+#define DECOR_LEFT		0x01
+#define DECOR_RIGHT		0x02
+#define DECOR_DOWN		0x04
+#define DECOR_UP		0x08
+#define DECOR_OPEN		0x10
+#define DECOR_CLOSED		0x20
+#define DECOR_BELOW		0x40
+
 /*
  * Structure defining the server configuration, including the map layout.
  */
