@@ -14,7 +14,7 @@
  *
  * This software is provided "as is" without any express or implied warranty.
  *
- * RCS:      $Id: socklib.c,v 3.49 1996/05/02 16:05:59 bert Exp $
+ * RCS:      $Id: socklib.c,v 3.60 1997/11/25 18:12:45 bert Exp $
  *
  * Revision 1.1.1.1  1992/05/11  12:32:34  bjoerns
  * XPilot v1.0
@@ -30,7 +30,7 @@
 
 #ifndef lint
 static char sourceid[] =
-    "@(#)$Id: socklib.c,v 3.49 1996/05/02 16:05:59 bert Exp $";
+    "@(#)$Id: socklib.c,v 3.60 1997/11/25 18:12:45 bert Exp $";
 #endif
 
 #ifdef TERMNET
@@ -42,14 +42,18 @@ static char sourceid[] =
 #define _SOCKLIB_LIBSOURCE
 
 /* Include files */
+#ifndef	_WINDOWS
 #include <unistd.h>
+#endif
 #include <sys/types.h>
 #ifdef VMS
 #include <ucx$inetdef.h>
 #else
+#ifndef	_WINDOWS
 #include <sys/param.h>
 #include <sys/ioctl.h>
 #endif
+#endif
 #if (SVR4)
 #include <sys/filio.h>
 #endif
@@ -58,19 +62,35 @@ static char sourceid[] =
 #else
 #include <fcntl.h>
 #endif
-#if defined(__hpux)
+#if defined(__hpux) || defined(_WINDOWS)
 #include <time.h>
 #else
 #include <sys/time.h>
 #endif
+
+#if defined(_WINDOWS)
+#include "../contrib/NT/xpilot/winNet.h"
+/* Windows needs specific system calls for sockets: */
+#undef close
+#define close(x__) closesocket(x__)
+#undef ioctl
+#define ioctl(x__, y__, z__) ioctlsocket(x__, y__, z__)
+#undef read
+#define read(x__, y__, z__) recv(x__, y__, z__,0)
+#undef write
+#define write(x__, y__, z__) send(x__, y__, z__,0)
+#else
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
+#include <netdb.h>
+#endif
+
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <netdb.h>
+
 #include <signal.h>
 #include <setjmp.h>
 #include <errno.h>
@@ -79,12 +99,6 @@ static char sourceid[] =
 #include <resolv.h>
 #endif
 
-#ifdef __cplusplus
-#ifndef __STDC__
-#define __STDC__	1
-#endif
-#endif
-
 /* Socklib Includes And Definitions */
 #include "version.h"
 #include "socklib.h"
@@ -124,12 +138,18 @@ int			sl_timeout_us = DEFAULT_US_TIMEOUT_VALUE;
 /* Global default retries variable used by DgramSendRec */
 int			sl_default_retries = DEFAULT_RETRIES;
 
-/* Global variable containing the last address from DgramReceiveAny */
-struct sockaddr_in	sl_dgram_lastaddr;
-
 /* Global broadcast enable variable (super-user only), default disabled */
 int			sl_broadcast_enabled = 0;
 
+/* Local variable containing the last address from DgramReceiveAny */
+static struct sockaddr_in	sl_dgram_lastaddr;
+
+#ifdef	_WINDOWS
+char	gethostbuf[MAXGETHOSTSTRUCT+1];
+BOOL	hostnameCancelled;
+BOOL	*hostnameFound;
+HANDLE	gethosthandle;
+#endif
 
 /*
  *******************************************************************************
@@ -162,13 +182,7 @@ int			sl_broadcast_enabled = 0;
  *
  * Originally coded by Arne Helme
  */
-void
-#ifdef __STDC__
-SetTimeout(int s, int us)
-#else
-SetTimeout(s, us)
-int s, us;
-#endif /* __STDC__ */
+void SetTimeout(int s, int us)
 {
     sl_timeout_us = us;
     sl_timeout_s = s;
@@ -208,13 +222,7 @@ int s, us;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-CreateServerSocket(int port)
-#else
-CreateServerSocket(port)
-int	port;
-#endif /* __STDC__ */
+int CreateServerSocket(int port)
 {
     struct sockaddr_in	addr_in;
     int			fd;
@@ -229,13 +237,13 @@ int	port;
     memset((char *)&addr_in, 0, sizeof(struct sockaddr_in));
     addr_in.sin_family		= AF_INET;
     addr_in.sin_addr.s_addr	= INADDR_ANY;
-    addr_in.sin_port		= htons(port);
+    addr_in.sin_port		= htons((u_short)port);
 
     retval = bind(fd, (struct sockaddr *)&addr_in, sizeof(struct sockaddr_in));
     if (retval < 0)
     {
 	sl_errno = SL_EBIND;
-	(void) close(fd);
+	close(fd);
 	return (-1);
     }
 
@@ -243,7 +251,7 @@ int	port;
     if (retval < 0)
     {
 	sl_errno = SL_ELISTEN;
-	(void) close(fd);
+	close(fd);
 	return (-1);
     }
 
@@ -280,13 +288,7 @@ int	port;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-GetPortNum(int fd)
-#else
-GetPortNum(fd)
-int	fd;
-#endif /* __STDC__ */
+int GetPortNum(int fd)
 {
     int			len;
     struct sockaddr_in	addr;
@@ -332,13 +334,7 @@ int	fd;
  *
  * Originally coded by Bert Gijsbers
  */
-char *
-#ifdef __STDC__
-GetSockAddr(int fd)
-#else
-GetSockAddr(fd)
-int	fd;
-#endif /* __STDC__ */
+char *GetSockAddr(int fd)
 {
     int			len;
     struct sockaddr_in	addr;
@@ -354,7 +350,7 @@ int	fd;
 /*
  *******************************************************************************
  *
- *	GetPeerName()
+ *	GetRemoteHostName()
  *
  *******************************************************************************
  * Description
@@ -381,17 +377,9 @@ int	fd;
  * Called By
  *	User applications
  *
- * Originally coded by Bert Gsbers
+ * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-SLGetPeerName(int fd, char *name, int namelen)
-#else
-SLGetPeerName(fd, name, namelen)
-int	fd;
-char	*name;
-int	namelen;
-#endif /* __STDC__ */
+int GetRemoteHostName(int fd, char *name, int namelen)
 {
     int			len;
     struct sockaddr_in	addr;
@@ -413,7 +401,7 @@ int	namelen;
     name[namelen - 1] = '\0';
 
     return (0);
-} /* GetPeerName */
+} /* GetRemoteHostName */
 
 
 /*
@@ -452,14 +440,7 @@ int	namelen;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-CreateClientSocket(char *host, int port)
-#else
-CreateClientSocket(host, port)
-char	*host;
-int	port;
-#endif /* __STDC__ */
+int CreateClientSocket(char *host, int port)
 {
     struct sockaddr_in	peer;
     struct hostent	*hp;
@@ -467,7 +448,7 @@ int	port;
 
     memset((char *)&peer, 0, sizeof(struct sockaddr_in));
     peer.sin_family = AF_INET;
-    peer.sin_port   = htons(port);
+    peer.sin_port   = htons((u_short)port);
 
     peer.sin_addr.s_addr = inet_addr(host);
     if (peer.sin_addr.s_addr == (int)-1)
@@ -492,7 +473,7 @@ int	port;
     if (connect(fd, (struct sockaddr *)&peer, sizeof(struct sockaddr_in)) < 0)
     {
 	sl_errno = SL_ECONNECT;
-	(void) close(fd);
+	close(fd);
 	return (-1);
     }
 
@@ -530,13 +511,7 @@ int	port;
  *
  * Originally coded by Arne Helme.
  */
-int
-#ifdef __STDC__
-SocketAccept(int fd)
-#else
-SocketAccept(fd)
-int	fd;
-#endif /* __STDC__ */
+int SocketAccept(int fd)
 {
     int		retval;
 
@@ -577,13 +552,7 @@ int	fd;
  *
  * Originally coded by Arne Helme, but moved out of SocketAccept by Bert.
  */
-int
-#ifdef __STDC__
-SocketLinger(int fd)
-#else
-SocketLinger(fd)
-int	fd;
-#endif /* __STDC__ */
+int SocketLinger(int fd)
 {
 #if defined(LINUX0) || !defined(SO_LINGER)
     /*
@@ -631,16 +600,9 @@ int	fd;
  * Called By
  *	User applications.
  *
- * Originally coded by Bert Gsbers
+ * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-SetSocketReceiveBufferSize(int fd, int size)
-#else
-SetSocketReceiveBufferSize(fd, size)
-int	fd;
-int	size;
-#endif /* __STDC__ */
+int SetSocketReceiveBufferSize(int fd, int size)
 {
     return (setsockopt(fd, SOL_SOCKET, SO_RCVBUF,
 		       (void *)&size, sizeof(size)));
@@ -675,16 +637,9 @@ int	size;
  * Called By
  *	User applications.
  *
- * Originally coded by Bert Gsbers
+ * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-SetSocketSendBufferSize(int fd, int size)
-#else
-SetSocketSendBufferSize(fd, size)
-int	fd;
-int	size;
-#endif /* __STDC__ */
+int SetSocketSendBufferSize(int fd, int size)
 {
     return (setsockopt(fd, SOL_SOCKET, SO_SNDBUF,
 		       (void *)&size, sizeof(size)));
@@ -719,17 +674,10 @@ int	size;
  * Called By
  *	User applications.
  *
- * Originally coded by Bert Gsbers
+ * Originally coded by Bert Gijsbers
  */
 #ifdef TCP_NODELAY
-int
-#ifdef __STDC__
-SetSocketNoDelay(int fd, int flag)
-#else
-SetSocketNoDelay(fd, flag)
-int	fd;
-int	flag;
-#endif /* __STDC__ */
+int SetSocketNoDelay(int fd, int flag)
 {
     /*
      * The fcntl(O_NDELAY) option has nothing to do
@@ -770,22 +718,15 @@ int	flag;
  * Called By
  *	User applications.
  *
- * Originally coded by Bert Gsbers
+ * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-SetSocketNonBlocking(int fd, int flag)
-#else
-SetSocketNonBlocking(fd, flag)
-int	fd;
-int	flag;
-#endif /* __STDC__ */
+int SetSocketNonBlocking(int fd, int flag)
 {
 /*
  * There are some problems on some particular systems (suns) with
  * getting sockets to be non-blocking.  Just try all possible ways
  * until one of them succeeds.  Please keep us informed by e-mail
- * to xpilot@cs.uit.no.
+ * to xpilot@xpilot.org.
  */
 
 #ifndef USE_FCNTL_O_NONBLOCK
@@ -837,7 +778,7 @@ int	flag;
 #endif
 
 #ifdef USE_IOCTL_FIONBIO
-    if (ioctl(fd, FIONBIO, &flag) != -1)
+    if (ioctl(fd, FIONBIO, &flag) == 0)
 	return 0;
     sprintf(buf, "ioctl FIONBIO failed in socklib.c line %d", __LINE__);
     perror(buf);
@@ -889,16 +830,9 @@ int	flag;
  * Called By
  *	User applications.
  *
- * Originally coded by Bert Gsbers
+ * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-SetSocketBroadcast(int fd, int flag)
-#else
-SetSocketBroadcast(fd, flag)
-int	fd;
-int	flag;
-#endif /* __STDC__ */
+int SetSocketBroadcast(int fd, int flag)
 {
     return setsockopt(fd, SOL_SOCKET, SO_BROADCAST,
 		      (void *)&flag, sizeof(flag));
@@ -932,15 +866,9 @@ int	flag;
  * Called By
  *	User applications.
  *
- * Originally coded by Bert Gsbers
+ * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-GetSocketError(int fd)
-#else
-GetSocketError(fd)
-int	fd;
-#endif /* __STDC__ */
+int GetSocketError(int fd)
 {
     int	error, size;
 
@@ -983,15 +911,9 @@ int	fd;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-SocketReadable(int fd)
-#else
-SocketReadable(fd)
-int	fd;
-#endif /* __STDC__ */
+int SocketReadable(int fd)
 {
-    int			readfds;
+    fd_set		readfds;
     struct timeval	timeout;
 
 #ifndef timerclear
@@ -1000,13 +922,19 @@ int	fd;
     timerclear(&timeout); /* macro function */
     timeout.tv_sec = sl_timeout_s;
     timeout.tv_usec = sl_timeout_us;
-    readfds = (1 << fd);
 
+#if a_very_bad_idea
+    memset(&readfds, 0, (fd / 8) + 1);
+#else
+    FD_ZERO(&readfds);
+    FD_SET(fd, &readfds);
+#endif
     if (select(fd + 1, &readfds, NULL, NULL, &timeout) == -1)
 	return ((errno == EINTR) ? 0 : -1);
 
-    if (readfds & (1 << fd))
+    if (FD_ISSET(fd, &readfds))
 	return (1);
+
     return (0);
 } /* SocketReadable */
 
@@ -1040,7 +968,7 @@ int	fd;
  *
  * Originally coded by Arne Helme
  */
-#ifdef __STDC__
+#if defined(__STDC__) || defined(__cplusplus) || defined(_WINDOWS)
 static void inthandler(int signum)
 #else
 static inthandler()
@@ -1085,33 +1013,38 @@ static inthandler()
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-SocketRead(int fd, char *buf, int size)
-#else
-SocketRead(fd, buf, size)
-int	fd, size;
-char	*buf;
-#endif /* __STDC__ */
+int SocketRead(int fd, char *buf, int size)
 {
     int	ret, ret1;
 
     if (setjmp(env))
     {
+#ifndef	_WINDOWS
 	(void) alarm(0);
 	(void) signal(SIGALRM, SIG_DFL);
+#else
+	alarm(0, NULL);
+#endif
 	return (-1);
     }
     ret = 0;
     cmw_priv_assert_netaccess();
     while (ret < size)
     {
+#ifndef	_WINDOWS
 	(void) signal(SIGALRM, inthandler);
 	(void) alarm(sl_timeout_s);
-	ret1 = read(fd, &buf[ret], size - ret);
+#else
+	(void) alarm(sl_timeout_s, inthandler);
+#endif
+	ret1 = recv(fd, &buf[ret], size - ret, 0);
 	DEB(fprintf(stderr, "Read %d bytes\n", ret1));
+#ifndef	_WINDOWS
 	(void) alarm(0);
 	(void) signal(SIGALRM, SIG_DFL);
+#else
+	(void) alarm(0, NULL);
+#endif
 	ret += ret1;
 	if (ret1 <= 0)
 	    return (-1);
@@ -1152,14 +1085,7 @@ char	*buf;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-SocketWrite(int fd, char *buf, int size)
-#else
-SocketWrite(fd, buf, size)
-int	fd, size;
-char	*buf;
-#endif /* __STDC__ */
+int SocketWrite(int fd, char *buf, int size)
 {
     int		retval;
 
@@ -1167,7 +1093,7 @@ char	*buf;
     /*
      * A SIGPIPE exception may occur if the peer entity has disconnected.
      */
-    retval = write(fd, buf, size);
+    retval = send(fd, buf, size, 0);
     cmw_priv_deassert_netaccess();
 
     return retval;
@@ -1206,13 +1132,7 @@ char	*buf;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-SocketClose(int fd)
-#else
-SocketClose(fd)
-int	fd;
-#endif /* __STDC__ */
+int SocketClose(int fd)
 {
     if (shutdown(fd, 2) == -1)
     {
@@ -1262,13 +1182,7 @@ int	fd;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-CreateDgramSocket(int port)
-#else
-CreateDgramSocket(port)
-int	port;
-#endif /* __STDC__ */
+int CreateDgramSocket(int port)
 {
     struct sockaddr_in	addr_in;
     int			fd;
@@ -1284,13 +1198,13 @@ int	port;
     memset((char *)&addr_in, 0, sizeof(struct sockaddr_in));
     addr_in.sin_family		= AF_INET;
     addr_in.sin_addr.s_addr	= INADDR_ANY;
-    addr_in.sin_port		= htons(port);
+    addr_in.sin_port		= htons((u_short)port);
     retval = bind(fd, (struct sockaddr *)&addr_in, sizeof(struct sockaddr_in));
     if (retval < 0)
     {
 	sl_errno = SL_EBIND;
 	retval = errno;
-	(void) close(fd);
+	close(fd);
 	errno = retval;
 	return (-1);
     }
@@ -1333,14 +1247,7 @@ int	port;
  *
  * Originally coded by Bert Gijsbers, adapted from CreateDgramSocket().
  */
-int
-#ifdef __STDC__
-CreateDgramAddrSocket(char *dotaddr, int port)
-#else
-CreateDgramAddrSocket(dotaddr, port)
-char	*dotaddr;
-int	port;
-#endif /* __STDC__ */
+int CreateDgramAddrSocket(char *dotaddr, int port)
 {
     struct sockaddr_in	addr_in;
     int			fd;
@@ -1356,13 +1263,13 @@ int	port;
     memset((char *)&addr_in, 0, sizeof(struct sockaddr_in));
     addr_in.sin_family		= AF_INET;
     addr_in.sin_addr.s_addr	= inet_addr(dotaddr);
-    addr_in.sin_port		= htons(port);
+    addr_in.sin_port		= htons((u_short)port);
     retval = bind(fd, (struct sockaddr *)&addr_in, sizeof(struct sockaddr_in));
     if (retval < 0)
     {
 	sl_errno = SL_EBIND;
 	retval = errno;
-	(void) close(fd);
+	close(fd);
 	errno = retval;
 	return (-1);
     }
@@ -1404,15 +1311,7 @@ int	port;
  *
  * Originally coded by Bert Gijsbers, adapted from CreateDgramAddrSocket().
  */
-int
-#ifdef __STDC__
-DgramBind(int fd, char *dotaddr, int port)
-#else
-DgramBind(fd, dotaddr, port)
-int	fd;
-char	*dotaddr;
-int	port;
-#endif /* __STDC__ */
+int DgramBind(int fd, char *dotaddr, int port)
 {
     struct sockaddr_in	addr_in;
     int			retval;
@@ -1420,7 +1319,7 @@ int	port;
     memset((char *)&addr_in, 0, sizeof(struct sockaddr_in));
     addr_in.sin_family		= AF_INET;
     addr_in.sin_addr.s_addr	= inet_addr(dotaddr);
-    addr_in.sin_port		= htons(port);
+    addr_in.sin_port		= htons((u_short)port);
     retval = bind(fd, (struct sockaddr *)&addr_in, sizeof(struct sockaddr_in));
     if (retval < 0)
     {
@@ -1462,17 +1361,9 @@ int	port;
  * Called By
  *	User applications.
  *
- * Originally coded by Bert Gsbers
+ * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-DgramConnect(int fd, char *host, int port)
-#else
-DgramConnect(fd, host, port)
-int	fd;
-char	*host;
-int	port;
-#endif /* __STDC__ */
+int DgramConnect(int fd, char *host, int port)
 {
     struct sockaddr_in	addr_in;
     struct hostent	*hp;
@@ -1493,7 +1384,7 @@ int	port;
 		((struct in_addr*)(hp->h_addr))->s_addr;
     }
     addr_in.sin_family		= AF_INET;
-    addr_in.sin_port		= htons(port);
+    addr_in.sin_port		= htons((u_short)port);
     retval = connect(fd, (struct sockaddr *)&addr_in, sizeof(addr_in));
     if (retval < 0)
     {
@@ -1542,15 +1433,7 @@ int	port;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-DgramSend(int fd, char *host, int port,
-	  char *sbuf, int size)
-#else
-DgramSend(fd, host, port, sbuf, size)
-int	fd, port, size;
-char	*host, *sbuf;
-#endif /* __STDC__ */
+int DgramSend(int fd, char *host, int port, char *sbuf, int size)
 {
     int			retval;
     struct sockaddr_in	the_addr;
@@ -1559,7 +1442,7 @@ char	*host, *sbuf;
     sl_errno = 0;
     (void) memset((char *)&the_addr, 0, sizeof(struct sockaddr_in));
     the_addr.sin_family		= AF_INET;
-    the_addr.sin_port		= htons(port);
+    the_addr.sin_port		= htons((u_short)port);
     if (sl_broadcast_enabled)
 	the_addr.sin_addr.s_addr	= INADDR_BROADCAST;
     else
@@ -1616,15 +1499,7 @@ char	*host, *sbuf;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-DgramReceiveAny(int fd, char *rbuf, int size)
-#else
-DgramReceiveAny(fd, rbuf, size)
-int	fd;
-char	*rbuf;
-int	size;
-#endif /* __STDC__ */
+int DgramReceiveAny(int fd, char *rbuf, int size)
 {
     int		retval;
     int		addrlen = sizeof(struct sockaddr_in);
@@ -1673,14 +1548,7 @@ int	size;
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-DgramReceive(int fd, char *from, char *rbuf, int size)
-#else
-DgramReceive(fd, from, rbuf, size)
-int	fd, size;
-char	*from, *rbuf;
-#endif /* __STDC__ */
+int DgramReceive(int fd, char *from, char *rbuf, int size)
 {
     struct sockaddr_in	tmp_addr;
     struct hostent	*hp;
@@ -1742,14 +1610,7 @@ char	*from, *rbuf;
  *
  * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-DgramReply(int fd, char *sbuf, int size)
-#else
-DgramReply(fd, sbuf, size)
-int	fd, size;
-char	*sbuf;
-#endif /* __STDC__ */
+int DgramReply(int fd, char *sbuf, int size)
 {
     int			retval;
 
@@ -1791,15 +1652,7 @@ char	*sbuf;
  *
  * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-DgramRead(int fd, char *rbuf, int size)
-#else
-DgramRead(fd, rbuf, size)
-int	fd;
-char	*rbuf;
-int	size;
-#endif /* __STDC__ */
+int DgramRead(int fd, char *rbuf, int size)
 {
     int		retval;
 
@@ -1841,15 +1694,7 @@ int	size;
  *
  * Originally coded by Bert Gijsbers
  */
-int
-#ifdef __STDC__
-DgramWrite(int fd, char *wbuf, int size)
-#else
-DgramWrite(fd, wbuf, size)
-int	fd;
-char	*wbuf;
-int	size;
-#endif /* __STDC__ */
+int DgramWrite(int fd, char *wbuf, int size)
 {
     int		retval;
 
@@ -1889,15 +1734,15 @@ int	size;
  *
  * Originally coded by Arne Helme
  */
-#ifdef __STDC__
-static void
-DgramInthandler(int signum)
+#if defined(__STDC__) || defined(__cplusplus) || defined(_WINDOWS)
+static void DgramInthandler(int signum)
 #else
-static
-DgramInthandler()
+static DgramInthandler()
 #endif /* __STDC__ */
 {
+#ifndef	_WINDOWS	/* hah?  This appears to do nothing. (unblock receive??) */
     (void) signal(SIGALRM, DgramInthandler);
+#endif
 } /* DgramInthandler */
 
 
@@ -1945,26 +1790,25 @@ DgramInthandler()
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-DgramSendRec(int fd, char *host, int port, char *sbuf,
+int DgramSendRec(int fd, char *host, int port, char *sbuf,
 	     int sbuf_size, char *rbuf, int rbuf_size)
-#else
-DgramSendRec(fd, host, port, sbuf, sbuf_size, rbuf, rbuf_size)
-int	fd, port, sbuf_size, rbuf_size;
-char	*host, *sbuf, *rbuf;
-#endif /* __STDC__ */
 {
     int		retval = -1;
     int		retry = sl_default_retries;
 
+#ifndef	_WINDOWS
     (void) signal(SIGALRM, DgramInthandler);
+#endif
     while (retry > 0)
     {
 	if (DgramSend(fd, host, port, sbuf, sbuf_size) == -1)
 	    return (-1);
 
+#ifndef	_WINDOWS
 	(void) alarm(sl_timeout_s);
+#else
+	(void) alarm(sl_timeout_s, DgramInthandler);
+#endif
 	retval = DgramReceive(fd, host, rbuf, rbuf_size);
 	if (retval == -1)
 	    if (errno == EINTR || sl_errno == SL_EWRONGHOST)
@@ -1984,8 +1828,12 @@ char	*host, *sbuf, *rbuf;
 	else
 	    break;		/* Datagram from <host> arrived */
     }
+#ifndef	_WINDOWS
     (void) alarm(0);
     (void) signal(SIGALRM, SIG_DFL);
+#else
+	alarm(0, NULL);
+#endif
     return (retval);
 } /* DgramInthandler */
 
@@ -2021,12 +1869,7 @@ char	*host, *sbuf, *rbuf;
  *
  * Originally coded by Arne Helme
  */
-char *
-#ifdef __STDC__
-DgramLastaddr(void)
-#else
-DgramLastaddr()
-#endif /* __STDC__ */
+char * DgramLastaddr(void)
 {
     return (inet_ntoa(sl_dgram_lastaddr.sin_addr));
 } /* DgramLastaddr */
@@ -2065,12 +1908,7 @@ DgramLastaddr()
  *
  * Originally coded by Bert Gijsbers
  */
-char *
-#ifdef __STDC__
-DgramLastname(void)
-#else
-DgramLastname()
-#endif /* __STDC__ */
+char *DgramLastname(void)
 {
     struct hostent	*he;
     char		*str;
@@ -2115,14 +1953,9 @@ DgramLastname()
  *
  * Originally coded by Arne Helme
  */
-int
-#ifdef __STDC__
-DgramLastport(void)
-#else
-DgramLastport()
-#endif /* __STDC__ */
+int DgramLastport(void)
 {
-    return (ntohs((int)sl_dgram_lastaddr.sin_port));
+    return ((int)ntohs((u_short)sl_dgram_lastaddr.sin_port));
 } /* DgramLastport */
 
 
@@ -2155,13 +1988,7 @@ DgramLastport()
  *
  * Originally coded by Bert Gijsbers
  */
-void
-#ifdef __STDC__
-DgramClose(int fd)
-#else
-DgramClose(fd)
-int	fd;
-#endif /* __STDC__ */
+void DgramClose(int fd)
 {
     close(fd);
 } /* DgramClose */
@@ -2200,15 +2027,13 @@ int	fd;
 #ifdef VMS
 #define MAXHOSTNAMELEN  256
 #endif
-#ifdef __STDC__
 void GetLocalHostName(char *name, unsigned size)
-#else
-void GetLocalHostName(name, size)
-    char		*name;
-    unsigned		size;
-#endif /* __STDC__ */
 {
-    struct hostent	*he, *xpilot_he, tmp;
+    struct hostent	*he = NULL;
+    struct hostent 	 *xpilot_he = NULL;
+#ifndef	_WINDOWS
+    struct hostent	  tmp;
+#endif
     int			xpilot_len;
     char		*alias, *dot;
     char		xpilot_hostname[MAXHOSTNAMELEN];
@@ -2223,14 +2048,14 @@ void GetLocalHostName(name, size)
 
     xpilot_len = strlen(xpilot);
 
+#ifndef	_WINDOWS	/* this takes FOREVER! zzzz */
     /* Make a wild guess that a "xpilot" hostname or alias is in this domain */
     if ((xpilot_he = gethostbyname(xpilot)) != NULL) {
-	if (strcmp(xpilot_he->h_name, "prince.mc.bio.uva.nl")) {
-	    strcpy(xpilot_hostname, xpilot_he->h_name);	/* copy data to buffer */
-	    tmp = *xpilot_he;
-	    xpilot_he = &tmp;
-	}
+	strcpy(xpilot_hostname, xpilot_he->h_name);	/* copy data to buffer */
+	tmp = *xpilot_he;
+	xpilot_he = &tmp;
     }
+#endif
 
     gethostname(name, size);
     if ((he = gethostbyname(name)) == NULL) {
@@ -2322,6 +2147,245 @@ void GetLocalHostName(name, size)
     }
 } /* GetLocalHostName */
 
+
+/*
+ *******************************************************************************
+ *
+ *	GetInetAddr()
+ *
+ *******************************************************************************
+ * Description
+ *	Convert a dotted decimal address to unsigned long.
+ *
+ * Input Parameters
+ *	Dotted decimal address.
+ *
+ * Output Parameters
+ *	None
+ *
+ * Return Value
+ *	The Internet address as unsigned long.
+ *
+ * Globals Referenced
+ *	None
+ *
+ * External Calls
+ *	inet_addr
+ *
+ * Called By
+ *	User applications.
+ *
+ * Originally coded by Bert Gijsbers
+ */
+unsigned long GetInetAddr(char *name)
+{
+    return inet_addr(name);
+}
+
+
+/*
+ *******************************************************************************
+ *
+ *	ResolveTimeout()
+ *
+ *******************************************************************************
+ * Description
+ *	Catch SIGALRM and do a longjmp.
+ *
+ * Input Parameters
+ *	The signal number.
+ *
+ * Output Parameters
+ *	None
+ *
+ * Return Value
+ *	None
+ *
+ * Globals Referenced
+ *	env
+ *
+ * External Calls
+ *	longjmp
+ *
+ * Called By
+ *	GetAddrByName and GetNameByAddr.
+ *
+ * Originally coded by Bert Gijsbers
+ */
+#if defined(__STDC__) || defined(__cplusplus) || defined(_WINDOWS)
+static void ResolveTimeout(int signum)
+#else
+static ResolveTimeout()
+#endif /* __STDC__ */
+{
+    DEB(fprintf(stderr, "Resolve timeout\n"));
+#ifdef	_WINDOWS
+	WSACancelAsyncRequest(gethosthandle);
+	hostnameCancelled=TRUE;
+	alarm(0,NULL);
+#else
+    (void) longjmp(env, 1);
+#endif
+}
+
+
+/*
+ *******************************************************************************
+ *
+ *	GetAddrByName()
+ *
+ *******************************************************************************
+ * Description
+ *	Convert a hostname to its dotted decimal address.
+ *
+ * Input Parameters
+ *	The name of the host.
+ *
+ * Output Parameters
+ *	None
+ *
+ * Return Value
+ *	On success a pointer to static memory containing the address.
+ *	On failure a NULL pointer.
+ *
+ * Globals Referenced
+ *	None
+ *
+ * External Calls
+ *	gethostbyname
+ *	signal
+ *	setjmp
+ *
+ * Called By
+ *	User applications.
+ *
+ * Originally coded by Bert Gijsbers
+ */
+#ifndef	_WINDOWS
+char *GetAddrByName(const char *name)
+{
+    struct hostent	*hp;
+
+    if (setjmp(env)) {
+	sl_errno = SL_ETIMEOUT;
+	alarm(0);
+	signal(SIGALRM, SIG_DFL);
+	return 0;
+    }
+    signal(SIGALRM, ResolveTimeout);
+    alarm(6);
+
+    hp = gethostbyname(name);
+
+    alarm(0);
+    signal(SIGALRM, SIG_DFL);
+    if (!hp) {
+	sl_errno = SL_EHOSTNAME;
+	return 0;
+    }
+    return inet_ntoa(*(struct in_addr *)(hp->h_addr));
+}
+#else
+char *GetAddrByName(const char *name)
+{
+	/* If you aren't connected to the net, then gethostbyname()
+	can take many minutes to time out.  WSACancelBlockingCall()
+	doesn't affect it.
+	*/
+	char	chp[MAXGETHOSTSTRUCT+1];
+	struct hostent* hp = (struct hostent*)&chp;
+	alarm(6, ResolveTimeout);
+	hostnameCancelled = FALSE;
+	*hostnameFound = FALSE;
+	gethosthandle = WSAAsyncGetHostByName(notifyWnd, WM_GETHOSTNAME, name, 
+								   chp, MAXGETHOSTSTRUCT);
+/*	hp = gethostbyname(name); */
+	while (!hostnameCancelled && !*hostnameFound)
+		Sleep(1000);
+	alarm(0, NULL);
+    if (!*hostnameFound)
+	{
+		sl_errno = SL_EHOSTNAME;
+		return 0;
+    }
+    return inet_ntoa(*(struct in_addr *)(hp->h_addr));
+}
+#endif
+
+/*
+ *******************************************************************************
+ *
+ *	GetNameByAddr()
+ *
+ *******************************************************************************
+ * Description
+ *	Convert a dotted decimal address to its hostname.
+ *
+ * Input Parameters
+ *	The dotted decimal address.
+ *
+ * Output Parameters
+ *	The name of the host.
+ *
+ * Return Value
+ *	On success a 0.
+ *	On failure a -1.
+ *
+ * Globals Referenced
+ *	None
+ *
+ * External Calls
+ *	gethostbyaddr
+ *	signal
+ *	setjmp
+ *
+ * Called By
+ *	User applications.
+ *
+ * Originally coded by Bert Gijsbers
+ */
+int GetNameByAddr(const char *addr, char *name, int size)
+{
+    struct hostent	*hp;
+    struct sockaddr_in	saddr;
+
+    saddr.sin_addr.s_addr = inet_addr(addr);
+    if (saddr.sin_addr.s_addr == (unsigned long)-1) {
+	sl_errno = SL_EADDR;
+	return -1;
+    }
+    if (setjmp(env)) {
+	sl_errno = SL_ETIMEOUT;
+#ifndef	_WINDOWS
+	signal(SIGALRM, SIG_DFL);
+#endif
+	return -1;
+    }
+#ifndef	_WINDOWS
+    alarm(0);
+    signal(SIGALRM, ResolveTimeout);
+    alarm(6);
+#else
+    alarm(0, NULL);
+    alarm(6, ResolveTimeout);
+#endif
+
+    hp = gethostbyaddr((char *)&saddr.sin_addr.s_addr, 4, AF_INET);
+#ifndef	_WINDOWS
+    alarm(0);
+    signal(SIGALRM, SIG_DFL);
+#else
+    alarm(0, NULL);
+#endif
+    if (!hp) {
+	sl_errno = SL_EHOSTNAME;
+	return -1;
+    }
+    strncpy(name, hp->h_name, size);
+    name[size - 1] = '\0';
+    return 0;
+}
+
 
 #if defined(__sun__)
 /*
