@@ -7,6 +7,8 @@
  * 29 July 1993
  *
  * You can do anything you like to this... :)
+ *
+ * $Id: strdup.c,v 3.4 1997/07/15 19:39:06 bert Exp $
  */
 
 #include <stdlib.h>
@@ -15,7 +17,11 @@
 char* strdup (const char* s1)
 {
 	char* s2;
-	if (s2 = (char*)malloc(strlen(s1)+1))
+
+	s2 = (char*)malloc(strlen(s1)+1);
+	if (s2)
 		strcpy(s2,s1);
+
 	return s2;
 }
+
