@@ -1,10 +1,11 @@
-/* $Id: update.c,v 3.70 1996/04/07 17:05:18 bert Exp $
+/* $Id: update.c,v 3.82 1998/01/08 19:28:53 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-97 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
+ *      Dick Balaska         <dick@xpilot.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,8 +22,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifdef	_WINDOWS
+#include "../contrib/NT/xpilots/winServer.h"
+#include <math.h>
+#include <stdio.h>
+#else
 #include <stdlib.h>
 #include <stdio.h>
+#endif
 
 #define SERVER
 #include "version.h"
@@ -34,25 +41,26 @@
 #include "score.h"
 #include "bit.h"
 #include "saudio.h"
+#include "objpos.h"
 
 char update_version[] = VERSION;
 
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: update.c,v 3.70 1996/04/07 17:05:18 bert Exp $";
+    "@(#)$Id: update.c,v 3.82 1998/01/08 19:28:53 bert Exp $";
 #endif
 
 
-#define update_object_speed(OBJ)	{				\
-    int x=(int)((OBJ)->pos.x/BLOCK_SZ), y=(int)((OBJ)->pos.y/BLOCK_SZ);	\
-    if (BIT((OBJ)->status, GRAVITY)) {					\
-	(OBJ)->vel.x += (OBJ)->acc.x + World.gravity[x][y].x;		\
-	(OBJ)->vel.y += (OBJ)->acc.y + World.gravity[x][y].y;		\
+#define update_object_speed(o_)						\
+    if (BIT((o_)->status, GRAVITY)) {					\
+	(o_)->vel.x += (o_)->acc.x					\
+		    + World.gravity[(o_)->pos.bx][(o_)->pos.by].x;	\
+	(o_)->vel.y += (o_)->acc.y					\
+		    + World.gravity[(o_)->pos.bx][(o_)->pos.by].y;	\
     } else {								\
-	(OBJ)->vel.x += (OBJ)->acc.x;					\
-	(OBJ)->vel.y += (OBJ)->acc.y;					\
-    }									\
-}
+	(o_)->vel.x += (o_)->acc.x;					\
+	(o_)->vel.y += (o_)->acc.y;					\
+    }
 
 
 static char msg[MSG_LEN];
@@ -69,7 +77,7 @@ static void Transport_to_home(int ind)
      * This results in a visually pleasing take off and landing.
      */
     player		*pl = Players[ind];
-    float		bx, by, dx, dy,	t, m;
+    DFLOAT		bx, by, dx, dy,	t, m;
     const int		T = RECOVERY_DELAY;
 
     if (BIT(World.rules->mode, TIMING) && pl->round) {
@@ -79,11 +87,11 @@ static void Transport_to_home(int ind)
 		check = pl->check - 1;
 	else
 		check = World.NumChecks - 1;
-	bx = World.check[check].x * BLOCK_SZ + BLOCK_SZ/2;
-	by = World.check[check].y * BLOCK_SZ + BLOCK_SZ/2;
+	bx = (World.check[check].x + 0.5) * BLOCK_SZ;
+	by = (World.check[check].y + 0.5) * BLOCK_SZ;
     } else {
-	bx = World.base[pl->home_base].pos.x * BLOCK_SZ + BLOCK_SZ/2;
-	by = World.base[pl->home_base].pos.y * BLOCK_SZ + BLOCK_SZ/2;
+	bx = (World.base[pl->home_base].pos.x + 0.5) * BLOCK_SZ;
+	by = (World.base[pl->home_base].pos.y + 0.5) * BLOCK_SZ;
     }
     dx = WRAP_DX(bx - pl->pos.x);
     dy = WRAP_DY(by - pl->pos.y);
@@ -200,14 +208,22 @@ static void do_Autopilot (player *pl)
     int		dir;
     int		afterburners;
     int		ix, iy;
-    float	gx, gy;
-    float	acc, vel;
-    float	delta;
-    float	turnspeed, power;
-    const float	emergency_thrust_settings_delta = 150.0 / FPS;
-    const float	auto_pilot_settings_delta = 15.0 / FPS;
-    const float	auto_pilot_turn_factor = 2.5;
-    const float	auto_pilot_dead_velocity = 0.5;
+    DFLOAT	gx, gy;
+    DFLOAT	acc, vel;
+    DFLOAT	delta;
+    DFLOAT	turnspeed, power;
+    const DFLOAT	emergency_thrust_settings_delta = 150.0 / FPS;
+    const DFLOAT	auto_pilot_settings_delta = 15.0 / FPS;
+    const DFLOAT	auto_pilot_turn_factor = 2.5;
+    const DFLOAT	auto_pilot_dead_velocity = 0.5;
+
+    /*
+     * If the last movement touched a wall then we shouldn't
+     * mess with the position (speed too?) settings.
+     */
+    if (pl->last_wall_touch + 1 >= frame_loops) {
+	return;
+    }
 
     /*
      * Having more autopilot items or using emergency thrust causes a much
@@ -227,8 +243,8 @@ static void do_Autopilot (player *pl)
 	afterburners = pl->item[ITEM_AFTERBURNER];
     }
 
-    ix = (int)(pl->pos.x / BLOCK_SZ);
-    iy = (int)(pl->pos.y / BLOCK_SZ);
+    ix = OBJ_X_IN_BLOCKS(pl);
+    iy = OBJ_Y_IN_BLOCKS(pl);
     gx = World.gravity[ix][iy].x;
     gy = World.gravity[ix][iy].y;
 
@@ -236,11 +252,11 @@ static void do_Autopilot (player *pl)
      * Due to rounding errors if the velocity is very small we were probably
      * on target to stop last time round, so we actually absolutely stop.
      * This enables the ship to orient away from gravity and set up the
-     * thrust to conteract it.
+     * thrust to counteract it.
      */
     if ((vel = LENGTH(pl->vel.x, pl->vel.y)) < auto_pilot_dead_velocity) {
 	pl->vel.x = pl->vel.y = vel = 0.0;
-	pl->pos = pl->prevpos;
+	Player_position_restore(pl);
     }
 
     /*
@@ -259,9 +275,9 @@ static void do_Autopilot (player *pl)
 	if (gx == 0 && gy == 0)
 	    vad = pl->dir;
 	else
-	    vad = findDir(-gx, -gy);
+	    vad = (int)findDir(-gx, -gy);
     } else {
-	vad = findDir(-pl->vel.x, -pl->vel.y);
+	vad = (int)findDir(-pl->vel.x, -pl->vel.y);
     }
     vad = MOD2(vad - pl->dir, RES);
     if (vad > RES/2) {
@@ -275,7 +291,7 @@ static void do_Autopilot (player *pl)
      * Calculate turnspeed needed to change direction instataneously by
      * above direction change.
      */
-    turnspeed = ((float)vad) / pl->turnresistance - pl->turnvel;
+    turnspeed = ((DFLOAT)vad) / pl->turnresistance - pl->turnvel;
     if (turnspeed < 0) {
 	turnspeed = -turnspeed;
 	dir = -dir;
@@ -353,10 +369,10 @@ static void do_Autopilot (player *pl)
 static void Tractor_beam (player *pl)
 {
     player		*victim;
-    float		maxdist, maxforce, percent;
-    float		xd, yd;
-    float		dvx, dvy;
-    float		force, mass;
+    DFLOAT		maxdist, maxforce, percent;
+    DFLOAT		xd, yd;
+    DFLOAT		dvx, dvy;
+    DFLOAT		force, mass;
     long		cost;
     int			theta;
 
@@ -371,7 +387,7 @@ static void Tractor_beam (player *pl)
 
     maxforce = TRACTOR_MAX_FORCE(pl);
     percent = TRACTOR_PERCENT(pl, maxdist);
-    cost = TRACTOR_COST(percent);
+    cost = (long)TRACTOR_COST(percent);
     force = TRACTOR_FORCE(pl, percent, maxforce);
 
     if (pl->fuel.sum < -cost) {
@@ -390,7 +406,7 @@ static void Tractor_beam (player *pl)
     xd = WRAP_DX(pl->pos.x - victim->pos.x);
     yd = WRAP_DY(pl->pos.y - victim->pos.y);
 
-    theta = findDir(xd, yd);
+    theta = (int)findDir(xd, yd);
     mass = pl->mass + victim->mass;
 
     dvx = tcos(theta) * (force / pl->mass);
@@ -449,7 +465,7 @@ void Update_objects(void)
      * Let the fuel stations regenerate some fuel.
      */
     if (NumPlayers > 0) {
-	int fuel = NumPlayers * STATION_REGENERATION;
+	int fuel = (int)(NumPlayers * STATION_REGENERATION);
 	int frames_per_update = MAX_STATION_FUEL / (fuel * BLOCK_SZ);
 	for (i=0; i<World.NumFuels; i++) {
 	    if (World.fuel[i].fuel == MAX_STATION_FUEL) {
@@ -577,6 +593,9 @@ void Update_objects(void)
 	LIMIT(pl->turnresistance, MIN_PLAYER_TURNRESISTANCE,
 				  MAX_PLAYER_TURNRESISTANCE);
 
+	if (pl->damaged > 0)
+	    pl->damaged--;
+
 	if (pl->count > 0) {
 	    pl->count--;
 	    if (!BIT(pl->status, PLAYING)) {
@@ -606,6 +625,11 @@ void Update_objects(void)
 	if (BIT(pl->status, PLAYING|GAME_OVER|PAUSE) != PLAYING)
 	    continue;
 
+	if (!cloakedShield && BIT(pl->used, OBJ_CLOAKING_DEVICE)) {
+	    CLR_BIT(pl->used, OBJ_EMERGENCY_SHIELD);
+	    CLR_BIT(pl->used, OBJ_SHIELD);
+	}
+
 	if (pl->shield_time > 0) {
 	    if (--pl->shield_time == 0) {
 		if (!BIT(pl->used, OBJ_EMERGENCY_SHIELD)) {
@@ -678,10 +702,10 @@ void Update_objects(void)
 	 * Compute energy drainage
 	 */
 	if (BIT(pl->used, OBJ_SHIELD))
-	    Add_fuel(&(pl->fuel), ED_SHIELD);
+	    Add_fuel(&(pl->fuel), (long)ED_SHIELD);
 
 	if (BIT(pl->used, OBJ_CLOAKING_DEVICE))
-	    Add_fuel(&(pl->fuel), ED_CLOAKING_DEVICE);
+	    Add_fuel(&(pl->fuel), (long)ED_CLOAKING_DEVICE);
 
 #define UPDATE_RATE 100
 
@@ -740,8 +764,8 @@ void Update_objects(void)
 	/* target repair */
 	if (BIT(pl->used, OBJ_REPAIR)) {
 	    target_t *targ = &World.targets[pl->repair_target];
-	    float x = targ->pos.x*BLOCK_SZ+BLOCK_SZ/2;
-	    float y = targ->pos.y*BLOCK_SZ+BLOCK_SZ/2;
+	    DFLOAT x = (targ->pos.x + 0.5) * BLOCK_SZ;
+	    DFLOAT y = (targ->pos.y + 0.5) * BLOCK_SZ;
 	    if (Wrap_length(pl->pos.x - x, pl->pos.y - y) > 90.0
 		|| targ->damage >= TARGET_DAMAGE
 		|| targ->dead_time > 0) {
@@ -783,12 +807,12 @@ void Update_objects(void)
 	 * Update acceleration vector etc.
 	 */
 	if (BIT(pl->status, THRUSTING)) {
-	    float power = pl->power;
-	    float f = pl->power * 0.0008;	/* 1/(FUEL_SCALE*MIN_POWER) */
+	    DFLOAT power = pl->power;
+	    DFLOAT f = pl->power * 0.0008;	/* 1/(FUEL_SCALE*MIN_POWER) */
 	    int a = (BIT(pl->used, OBJ_EMERGENCY_THRUST)
 		     ? MAX_AFTERBURNER
 		     : pl->item[ITEM_AFTERBURNER]);
-	    float inert = pl->mass;
+	    DFLOAT inert = pl->mass;
 
 	    if (a) {
 		power = AFTER_BURN_POWER(power, a);
@@ -796,7 +820,7 @@ void Update_objects(void)
 	    }
 	    pl->acc.x = power * tcos(pl->dir) / inert;
 	    pl->acc.y = power * tsin(pl->dir) / inert;
-	    Add_fuel(&(pl->fuel), -f * FUEL_SCALE_FACT); /* Decrement fuel */
+	    Add_fuel(&(pl->fuel), (long)(-f * FUEL_SCALE_FACT)); /* Decrement fuel */
 	} else {
 	    pl->acc.x = pl->acc.y = 0.0;
 	}
@@ -812,7 +836,7 @@ void Update_objects(void)
 	    if (World.wormHoles[pl->wormHoleHit].countdown
 		&& World.wormHoles[pl->wormHoleHit].lastplayer != i)
 		j = World.wormHoles[pl->wormHoleHit].lastdest;
-	    else if (rand() % 100 < 10)
+	    else if (rfrac() < 0.10f)
 		do
 		    j = rand() % World.NumWormholes;
 		while (World.wormHoles[j].type == WORM_IN
@@ -833,7 +857,7 @@ void Update_objects(void)
 		    wx = WRAP_DX(wx);
 		    wy = WRAP_DX(wy);
 
-		    proximity = pl->vel.y * wx + pl->vel.x * wy;
+		    proximity = (int)(pl->vel.y * wx + pl->vel.x * wy);
 		    proximity = ABS(proximity);
 
 		    if (pl->vel.x * wx + pl->vel.y * wy < 0) {
@@ -863,8 +887,8 @@ void Update_objects(void)
 
 	    sound_play_sensors(pl->pos.x, pl->pos.y, WORM_HOLE_SOUND);
 
-	    w.x = World.wormHoles[j].pos.x * BLOCK_SZ + BLOCK_SZ / 2;
-	    w.y = World.wormHoles[j].pos.y * BLOCK_SZ + BLOCK_SZ / 2;
+	    w.x = (World.wormHoles[j].pos.x + 0.5) * BLOCK_SZ;
+	    w.y = (World.wormHoles[j].pos.y + 0.5) * BLOCK_SZ;
 
 	    /*
 	     * Don't connect to balls while warping.
@@ -873,32 +897,38 @@ void Update_objects(void)
 		pl->ball = NULL;
 
 	    if (BIT(pl->have, OBJ_BALL)) {
-		object	*b;
-		int k;
-
 		/*
 		 * Take every ball associated with player through worm hole.
 		 * NB. the connector can cross a wall boundary this is
 		 * allowed, so long as the ball itself doesn't collide.
 		 */
+		int k;
 		for (k = 0; k < NumObjs; k++) {
-		    if (!BIT(Obj[k]->type, OBJ_BALL) || Obj[k]->id != pl->id)
-			continue;
-		    b = Obj[k];
-
-		    b->pos.x = w.x + b->pos.x - pl->pos.x;
-		    b->pos.y = w.y + b->pos.y - pl->pos.y;
-		    b->vel.x /= WORM_BRAKE_FACTOR;
-		    b->vel.y /= WORM_BRAKE_FACTOR;
-		    b->prevpos = b->pos;
+		    object *b = Obj[k];
+		    if (BIT(b->type, OBJ_BALL) && b->id == pl->id) {
+			position ballpos;
+			ballpos.x = b->pos.x + (w.x - pl->pos.x);
+			ballpos.y = b->pos.y + (w.y - pl->pos.y);
+			ballpos.x = WRAP_XPIXEL(ballpos.x);
+			ballpos.y = WRAP_YPIXEL(ballpos.y);
+			if (ballpos.x < 0 || ballpos.x >= World.width
+			    || ballpos.y < 0 || ballpos.y >= World.height) {
+			    b->life = 0;
+			}
+			else {
+			    Object_position_set_pixels(b, ballpos.x, ballpos.y);
+			    Object_position_remember(b);
+			    b->vel.x *= WORM_BRAKE_FACTOR;
+			    b->vel.y *= WORM_BRAKE_FACTOR;
+			}
+		    }
 		}
 	    }
 
 	    pl->wormHoleDest = j;
-	    pl->pos = w;
-	    pl->vel.x /= WORM_BRAKE_FACTOR;
-	    pl->vel.y /= WORM_BRAKE_FACTOR;
-	    pl->prevpos = pl->pos;
+	    Player_position_init_pixels(pl, w.x, w.y);
+	    pl->vel.x *= WORM_BRAKE_FACTOR;
+	    pl->vel.y *= WORM_BRAKE_FACTOR;
 	    pl->forceVisible += 15;
 
 	    if (j != pl->wormHoleHit) {
@@ -928,7 +958,7 @@ void Update_objects(void)
 	    if (necm < MAX_PLAYER_ECMS) {
 		pl->ecmInfo.pos[necm].x = pl->pos.x;
 		pl->ecmInfo.pos[necm].y = pl->pos.y;
-		pl->ecmInfo.size[necm] = ECM_DISTANCE;
+		pl->ecmInfo.size[necm] = (int)ECM_DISTANCE;
 		pl->ecmInfo.count++;
 	    }
 	    CLR_BIT(pl->used, OBJ_ECM);
@@ -943,7 +973,9 @@ void Update_objects(void)
 	}
 
 	if (BIT(pl->status, THRUSTING)) {
-	    Thrust(i);
+	    if (!BIT(pl->used, OBJ_CLOAKING_DEVICE) || cloakedExhaust) {
+  		Thrust(i);
+	    }
 	}
 #ifdef TURN_FUEL
 	if (tf)
