@@ -1,10 +1,10 @@
-/* $Id: xp-replay.c,v 3.25 1996/05/02 16:06:03 bert Exp $
+/* $Id: xp-replay.c,v 3.30 1997/11/27 20:09:43 bert Exp $
  *
  * XP-Replay, playback an XPilot session.  Copyright (C) 1994-95 by
  *
- *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
- *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
- *      Bert Gsbers         (bert@mc.bio.uva.nl)
+ *      Bjrn Stabell        <bjoern@xpilot.org>
+ *      Ken Ronny Schouten   <ken@xpilot.org>
+ *      Bert Gijsbers        <bert@xpilot.org>
  *      Steven Singer        (S.Singer@ph.surrey.ac.uk)
  *
  * This program is free software; you can redistribute it and/or modify
@@ -297,7 +297,7 @@ struct label {
     int			x, y;		/* position */
     int			w;		/* width (used to line up values) */
     Bool		j;		/* True for right justification */
-    char		*name;		/* Field name */
+    const char		*name;		/* Field name */
     enum LabelDataTypes	type;		/* data type */
     union {
 	int		*i;
@@ -438,7 +438,7 @@ static long	mem_all_types_used;		/* frame memory in use */
 static long	mem_typed_used[NUM_MEMTYPES];	/* debugging & analysis */
 static long	max_mem = 8 * 1024 * 1024;	/* memory limit (soft) */
 
-static void openErrorWindow(struct errorwin *, char *, ...);
+static void openErrorWindow(struct errorwin *, const char *, ...);
 
 /*
  * Print memory statistics when debugging.
@@ -619,7 +619,7 @@ static char *RReadString(FILE *fp)
 {
     char		*s;
     int			i;
-    size_t		len;
+    int			len;
 
     len = RReadUShort(fp);
     s = (char *) MyMalloc(len + 1, MEM_STRING);
@@ -1380,7 +1380,7 @@ static char		*itemData[NUM_ITEMS] = {
     itemEmergencyShield_bits
 };
 
-static XFontStruct *loadQueryFont(char *fontName, GC gc)
+static XFontStruct *loadQueryFont(const char *fontName, GC gc)
 {
     XFontStruct		*font;
 
@@ -1448,7 +1448,7 @@ static void allocViewColors(struct xprc *rc)
     }
 }
 
-static unsigned long allocColor(char *name)
+static unsigned long allocColor(const char *name)
 {
     XColor		color;
 
@@ -1669,6 +1669,7 @@ static void Init_wm_prop(Window win,
     XWMHints		xwmh;
     XSizeHints		xsh;
     char		msg[256];
+    static char		myClass[] = "XP-replay";
 
     xwmh.flags	   = InputHint|StateHint;
     xwmh.input	   = True;
@@ -1689,7 +1690,7 @@ static void Init_wm_prop(Window win,
     xsh.y = y;
 
     xclh.res_name = NULL;		/* NULL: Automatically uses Argv[0], */
-    xclh.res_class = "XP-replay";	/* stripped of directory prefixes. */
+    xclh.res_class = myClass;		/* stripped of directory prefixes. */
 
     /*
      * Set the above properties.
@@ -1814,7 +1815,7 @@ static struct recordwin *Init_recordwindow(unsigned long bg, void *data)
     return(rwin);
 }
 
-static void openErrorWindow(struct errorwin *ewin, char *fmt, ...)
+static void openErrorWindow(struct errorwin *ewin, const char *fmt, ...)
 {
     int w, h, i;
     va_list ap;
@@ -2266,12 +2267,12 @@ static void ScalePPM(unsigned char *rgbdata, int cols, int rows,
 	BuildGamma(gammatbl, gamma);
     }
 
-    newcols = cols * scale + 0.999;
-    newrows = rows * scale + 0.999;
+    newcols = (int) (cols * scale + 0.999);
+    newrows = (int) (rows * scale + 0.999);
     xscale = (double) newcols / (double) cols;
     yscale = (double) newrows / (double) rows;
-    sxscale = xscale * SCALE;
-    syscale = yscale * SCALE;
+    sxscale = (long) (xscale * SCALE);
+    syscale = (long) (yscale * SCALE);
     size_newxelrow = 3 * newcols;
     size_tempxelrow = 3 * cols;
     size_rsgsbs = cols * sizeof(long);
@@ -3216,12 +3217,15 @@ static void dox(struct xui *ui, struct xprc *rc)
 	    long	frame_rate = 1000000L / rc->fps;
 
 	    if (delta_time + 1000 < frame_rate) {
+		fd_set	rset;
 		int	rfd = ConnectionNumber(dpy);
 		int	num;
 
 		tv1.tv_sec = 0;
 		tv1.tv_usec = frame_rate - delta_time;
-		num = select(rfd + 1, &rfd, NULL, NULL, &tv1);
+		FD_ZERO(&rset);
+		FD_SET(rfd, &rset);
+		num = select(rfd + 1, &rset, NULL, NULL, &tv1);
 		if (num == 1) {
 		    continue;
 		}
